@@ -12,6 +12,10 @@
 #include "AVSMap.h"
 #include "parser/scriptparser.h"
 
+#ifndef MINGW_HAS_SECURE_API
+#define sprintf_s sprintf
+#endif
+
 #define ENABLE_CUDA_COMPUTE_STREAM 0
 
 #ifdef ENABLE_CUDA
@@ -63,11 +67,9 @@ std::string DeviceTypesString(int devicetypes)
   if (devicetypes & DEV_TYPE_CPU) {
     typesstr.push_back("CPU");
   }
-#ifdef ENABLE_CUDA
   if (devicetypes & DEV_TYPE_CUDA) {
     typesstr.push_back("CUDA");
   }
-#endif
   std::ostringstream oss;
   for (int i = 0; i < (int)typesstr.size(); ++i) {
     if (i > 0) oss << ",";
@@ -85,6 +87,7 @@ static void CheckDeviceTypes(const char* name, int devicetypes, const AVSValue&
       if ((devicetypes & childtypes) == 0) {
         std::string parentdevstr = DeviceTypesString(devicetypes);
         std::string childdevstr = DeviceTypesString(childtypes);
+        // e.g.. Device unmatch: XYfilter[CPU] does not support [CUDA] frame
         env->ThrowError(
           "Device unmatch: %s[%s] does not support [%s] frame",
           name, parentdevstr.c_str(), childdevstr.c_str());
@@ -374,7 +377,7 @@ public:
     if (mem > 0) {
       uint64_t requested = mem * 1048576ull;
       uint64_t mem_limit = prop.totalGlobalMem;
-      memory_max = clamp(requested, 64 * 1024 * 1024ull, mem_limit - 128 * 1024 * 1024ull);
+      memory_max = clamp(requested, (uint64_t)(64 * 1024 * 1024ull), (uint64_t)(mem_limit - 128 * 1024 * 1024ull));
     }
     return (int)(memory_max / 1048576ull);
   }
@@ -462,11 +465,31 @@ DeviceManager::DeviceManager(InternalEnvironment* env) :
 
 #ifdef ENABLE_CUDA
   int cuda_device_count = 0;
-  if (cudaGetDeviceCount(&cuda_device_count) == cudaSuccess) {
+  cudaError_t status = cudaGetDeviceCount(&cuda_device_count);
+  if (status == cudaSuccess) {
+    // _RPT0(0, "cudaGetDeviceCount = %d\r\n", cuda_device_count);
     for (int i = 0; i < cuda_device_count; ++i) {
       cudaDevices.emplace_back(new CUDADevice(next_device_id++, i, env));
     }
   }
+  else {
+    if (status == cudaErrorInitializationError) {
+      // We probably get this error because of current Nvidia driver version is lower than
+      // the minimum required version by the used CUDA SDK, or a Computing Capability is too low.
+      // Example: GTX460: Latest driver is 391.35, latest supporting CUDA SDK is 9.1.85
+      // Consequence: since SDK 9 is unsupported in VS2019, no GTX460 support
+      // As of Jan.2021 Avisynth+ is is using CUDA Toolkit version 11.2.
+      int version;
+      status = cudaRuntimeGetVersion(&version);
+      if(status == cudaSuccess)
+        _RPT1(0, "cudaGetDeviceCount: cudaErrorInitializationError!\r\nMaybe CUDA Runtime version (%d) does not support old drivers. \r\n", version);
+      else
+        _RPT0(0, "cudaGetDeviceCount: cudaErrorInitializationError! Runtime version request failed\r\n");
+    }
+    else {
+      _RPT1(0, "cudaGetDeviceCount failed (%d)\r\n", (int)status);
+    }
+  }
   // do not modify CUDADevices after this since it causes pointer change
 
   cpuDevice = std::unique_ptr<Device>((cuda_device_count > 0)
@@ -486,8 +509,8 @@ Device* DeviceManager::GetDevice(AvsDeviceType device_type, int device_index) co
   case DEV_TYPE_CPU:
     return cpuDevice.get();
 
-#ifdef ENABLE_CUDA
   case DEV_TYPE_CUDA:
+#ifdef ENABLE_CUDA
     if (device_index < 0) {
       env->ThrowError("Invalid device index %d", device_index);
     }
@@ -497,7 +520,9 @@ Device* DeviceManager::GetDevice(AvsDeviceType device_type, int device_index) co
     // wrap index
     device_index %= (int)cudaDevices.size();
     return cudaDevices[device_index].get();
-#endif // #ifdef ENABLE_CUDA
+#else
+    env->ThrowError("This Avisynth does not support memory type %d (CUDA)", device_type);
+#endif
 
   default:
     env->ThrowError("Not supported memory type %d", device_type);
@@ -512,10 +537,8 @@ int DeviceManager::GetNumDevices(AvsDeviceType device_type) const
   case DEV_TYPE_CPU:
     return 1;
 
-#ifdef ENABLE_CUDA
   case DEV_TYPE_CUDA:
     return (int)cudaDevices.size();
-#endif // #ifdef ENABLE_CUDA
 
   default:
     env->ThrowError("Not supported memory type %d", device_type);
@@ -828,18 +851,18 @@ class CUDAFrameTransferEngine : public FrameTransferEngine
     PVideoFrame dst = env->GetOnDeviceFrame(src, downstreamDevice);
     TransferFrameData(dst, src, false, env);
 
-#ifndef NEOFP
+#if 1
     AVSMap* mapv = env->getFramePropsRW(dst);
     const int numKeys = env->propNumKeys(mapv);
     for (int i = 0; i < numKeys; i++) {
       const char* key = env->propGetKey(mapv, i);
-      if (env->propGetType(mapv, key) == 'v') {
+      if (env->propGetType(mapv, key) == AVSPropTypes::PROPTYPE_FRAME) {
         // isFrame true
         const int numElements = env->propNumElements(mapv, key);
 
         std::vector<PVideoFrame> frameset;
         int error;
-
+        // avs+: can be more frames in a frame property array
         for (int index = 0; index < numElements; index++) {
           const PVideoFrame srcframe = env->propGetFrame(mapv, key, index, &error);
           frameset.push_back(srcframe);
@@ -851,11 +874,12 @@ class CUDAFrameTransferEngine : public FrameTransferEngine
           PVideoFrame src = frameset[index];
           PVideoFrame dst = env->GetOnDeviceFrame(src, downstreamDevice);
           TransferFrameData(dst, src, false, env);
-          env->propSetFrame(mapv, key, dst, AVSPropAppendMode::paAppend);
+          env->propSetFrame(mapv, key, dst, AVSPropAppendMode::PROPAPPENDMODE_APPEND);
         }
       }
     }
 #else
+    // kept for reference from neo fork, a single frame in frame properties. No array of frames here
     AVSMap* mapv = env->GetAVSMap(dst);
     for (auto it = mapv->data.begin(), end = mapv->data.end(); it != end; ++it) {
       if (it->second.IsFrame()) {
@@ -885,18 +909,18 @@ class CUDAFrameTransferEngine : public FrameTransferEngine
 
     TransferFrameData(cacheHandle.first->value, item.src, true, env);
 
-#ifndef NEOFP
+#if 1
     AVSMap* mapv = env->getFramePropsRW(cacheHandle.first->value);
     const int numKeys = env->propNumKeys(mapv);
     for (int i = 0; i < numKeys; i++) {
       const char* key = env->propGetKey(mapv, i);
-      if (env->propGetType(mapv, key) == 'v') {
+      if (env->propGetType(mapv, key) == AVSPropTypes::PROPTYPE_FRAME) {
         // isFrame true
         const int numElements = env->propNumElements(mapv, key);
 
         std::vector<PVideoFrame> frameset;
         int error;
-
+        // avs+: can be more frames in a frame property array
         for (int index = 0; index < numElements; index++) {
           const PVideoFrame srcframe = env->propGetFrame(mapv, key, index, &error);
           frameset.push_back(srcframe);
@@ -908,11 +932,12 @@ class CUDAFrameTransferEngine : public FrameTransferEngine
           PVideoFrame src = frameset[index];
           PVideoFrame dst = env->GetOnDeviceFrame(src, downstreamDevice);
           TransferFrameData(dst, src, true, env);
-          env->propSetFrame(mapv, key, dst, AVSPropAppendMode::paAppend);
+          env->propSetFrame(mapv, key, dst, AVSPropAppendMode::PROPAPPENDMODE_APPEND);
         }
       }
     }
 #else
+    // kept for reference from neo fork, a single frame in frame properties. No frame arrays here
     AVSMap* mapv = env->GetAVSMap(cacheHandle.first->value);
     for (auto it = mapv->data.begin(), end = mapv->data.end(); it != end; ++it) {
       if (it->second.IsFrame()) {
@@ -1233,9 +1258,7 @@ public:
     if (args[0].IsClip()) {
       PClip clip = args[0].AsClip();
       int numPrefetch = args[1].Defined() ? args[1].AsInt() : 1;
-#ifdef ENABLE_CUDA
       int upstreamIndex = (args.ArraySize() >= 3 && args[2].Defined()) ? args[2].AsInt() : 0;
-#endif
 
       if (numPrefetch < 0) {
         numPrefetch = 0;
@@ -1244,32 +1267,27 @@ public:
       switch (upstreamType) {
       case DEV_TYPE_CPU:
         return new OnDevice(clip, numPrefetch, (Device*)(void*)env->GetDevice(DEV_TYPE_CPU, 0), env);
-#ifdef ENABLE_CUDA
       case DEV_TYPE_CUDA:
         return new OnDevice(clip, numPrefetch, (Device*)(void*)env->GetDevice(DEV_TYPE_CUDA, upstreamIndex), env);
-#endif
+      default:
+        env->ThrowError("Not supported device ...");
+        return AVSValue();
       }
 
-      env->ThrowError("Not supported device ...");
-      return AVSValue();
     }
     else {
       assert(args[0].IsFunction());
       PFunction func = args[0].AsFunction();
-#ifdef ENABLE_CUDA
       int upstreamIndex = (args.ArraySize() >= 2 && args[1].Defined()) ? args[1].AsInt() : 0;
-#endif
 
       Device* upstreamDevice = nullptr;
       switch (upstreamType) {
       case DEV_TYPE_CPU:
         upstreamDevice = (Device*)(void*)env->GetDevice(DEV_TYPE_CPU, 0);
         break;
-#ifdef ENABLE_CUDA
       case DEV_TYPE_CUDA:
         upstreamDevice = (Device*)(void*)env->GetDevice(DEV_TYPE_CUDA, upstreamIndex);
         break;
-#endif
       default:
         env->ThrowError("Not supported device ...");
         break;
@@ -1335,12 +1353,8 @@ PVideoFrame GetFrameOnDevice(PClip& c, int n, const PDevice& device, InternalEnv
 
 extern const AVSFunction Device_filters[] = {
   { "OnCPU", BUILTIN_FUNC_PREFIX, "c[num_prefetch]i", OnDevice::Create, (void*)DEV_TYPE_CPU },
-#ifdef ENABLE_CUDA
   { "OnCUDA", BUILTIN_FUNC_PREFIX, "c[num_prefetch]i[device_index]i", OnDevice::Create, (void*)DEV_TYPE_CUDA },
-#endif
   { "OnCPU", BUILTIN_FUNC_PREFIX, "n", OnDevice::Create, (void*)DEV_TYPE_CPU },
-#ifdef ENABLE_CUDA
   { "OnCUDA", BUILTIN_FUNC_PREFIX, "n[device_index]i", OnDevice::Create, (void*)DEV_TYPE_CUDA },
-#endif
   { 0 }
 };
