@@ -163,7 +163,6 @@ PVideoFrame __stdcall MTGuard::GetFrame(int n, IScriptEnvironment* env)
   }
   case MT_MULTI_INSTANCE:
   {
-    auto myThreadID = envI->GetThreadId();
     // When called from thread pool then thread IDs are one-to-one mapped to ChildFilters array.
     // 'binary and' mask (old method) or 'modulo' method ensures that no over-addressing can happen.
     // The number of filter instances are created in EnableMT as such.
@@ -175,7 +174,9 @@ PVideoFrame __stdcall MTGuard::GetFrame(int n, IScriptEnvironment* env)
     //   the mapping is not ideal. It can be less or more than the actual instance count.
     //   E.g. when mapping a thread on a CPU with 8 logical cores 
     //   to a instance count of 3 (nThread=3) the mapping is uneven (modulo example): [0,3,6]->[0] [1,4,7]->[1] [2,5]->[2]
+    // Resolution: do not use OLD_PREFETCH source part (kept just for knowledge)
 #ifdef OLD_PREFETCH
+    auto myThreadID = envI->GetThreadId();
     assert((nThreads & (nThreads - 1)) == 0); // must be 2^n
     size_t clipIndex = myThreadID & (nThreads - 1);
 #else
@@ -234,12 +235,13 @@ void __stdcall MTGuard::GetAudio(void* buf, int64_t start, int64_t count, IScrip
 {
   assert(nThreads > 0);
 
+  /* commented out, see MTGuard::GetFrame comments for MT_SERIALIZED
   if (nThreads == 1)
   {
     ChildFilters[0].filter->GetAudio(buf, start, count, env);
     return;
   }
-
+  */
   InternalEnvironment *envI = static_cast<InternalEnvironment*>(env);
 
   switch (MTMode)
@@ -251,7 +253,14 @@ void __stdcall MTGuard::GetAudio(void* buf, int64_t start, int64_t count, IScrip
     }
   case MT_MULTI_INSTANCE:
     {
-      auto& child = ChildFilters[envI->GetThreadId() & (nThreads - 1)];
+#ifdef OLD_PREFETCH
+      auto myThreadID = envI->GetThreadId();
+      assert((nThreads & (nThreads - 1)) == 0); // must be 2^n
+      size_t clipIndex = myThreadID & (nThreads - 1);
+#else
+      size_t clipIndex = envI->GetThreadId() % nThreads;
+#endif
+      auto& child = ChildFilters[clipIndex];
       std::lock_guard<std::mutex> lock(child.mutex);
       child.filter->GetAudio(buf, start, count, env);
       break;
@@ -294,6 +303,11 @@ int __stdcall MTGuard::SetCacheHints(int cachehints, int frame_range)
   if (CACHE_IS_MTGUARD_REQ == cachehints) {
     return CACHE_IS_MTGUARD_ANS;
   }
+  // Filter-MTGuard-CacheGuard(Cache or nothing)
+  // MTGuard is just a helper over real filters, we pass these requests upstream
+  if (CACHE_GET_AUDIO_POLICY == cachehints || CACHE_GET_AUDIO_SIZE == cachehints ||
+    CACHE_GETCHILD_AUDIO_MODE == cachehints || CACHE_GETCHILD_AUDIO_SIZE == cachehints)
+    return (ChildFilters[0].filter->GetVersion() >= 5) ? ChildFilters[0].filter->SetCacheHints(cachehints, 0) : 0;
   if (CACHE_GET_DEV_TYPE == cachehints || CACHE_GET_CHILD_DEV_TYPE == cachehints) {
     return (ChildFilters[0].filter->GetVersion() >= 5) ? ChildFilters[0].filter->SetCacheHints(cachehints, 0) : 0;
   }
