@@ -312,10 +312,10 @@ const BYTE * AVSC_CC avs_get_read_ptr_p(const AVS_VideoFrame * p, int plane)
 {
   switch (plane) {
   case AVS_PLANAR_U: case AVS_PLANAR_B: return p->vfb->data + p->offsetU; // G is first. Then B,R order like U,V
-  case AVS_PLANAR_V: case PLANAR_R:     return p->vfb->data + p->offsetV;
-  case PLANAR_A: return p->vfb->data + p->offsetA;
+  case AVS_PLANAR_V: case AVS_PLANAR_R: return p->vfb->data + p->offsetV;
+  case AVS_PLANAR_A: return p->vfb->data + p->offsetA;
   default:           return p->vfb->data + p->offset;
-  } // PLANAR Y, PLANAR_G
+  } // AVS_PLANAR Y, AVS_PLANAR_G
 }
 
 extern "C"
@@ -338,6 +338,19 @@ int AVSC_CC avs_is_property_writable(const AVS_VideoFrame * p)
   return 0;
 }
 
+// V10
+extern "C"
+int AVSC_CC avs_video_frame_get_pixel_type(const AVS_VideoFrame * p)
+{
+  return (*(const PVideoFrame*)&p)->GetPixelType();
+}
+
+// V10
+void AVSC_CC avs_video_frame_amend_pixel_type(AVS_VideoFrame* p, int new_pixel_type)
+{
+  (*(PVideoFrame*)&p)->AmendPixelType(new_pixel_type);
+}
+
 extern "C"
 BYTE * AVSC_CC avs_get_write_ptr_p(const AVS_VideoFrame * p, int plane)
 {
@@ -386,6 +399,25 @@ int AVSC_CC avs_bits_per_component(const AVS_VideoInfo * p)
   return ((VideoInfo*)p)->BitsPerComponent();
 }
 
+// V10.1
+extern "C"
+bool AVSC_CC avs_is_channel_mask_known(const AVS_VideoInfo * p)
+{
+  return ((VideoInfo*)p)->IsChannelMaskKnown();
+}
+
+extern "C"
+void AVSC_CC avs_set_channel_mask(const AVS_VideoInfo * p, bool isChannelMaskKnown, unsigned int dwChannelMask)
+{
+  ((VideoInfo*)p)->SetChannelMask(isChannelMaskKnown, dwChannelMask);
+}
+
+extern "C"
+unsigned int AVSC_CC avs_get_channel_mask(const AVS_VideoInfo * p)
+{
+  return ((VideoInfo*)p)->GetChannelMask();
+}
+
 //////////////////////////////////////////////////////////
 //
 // frame properties
@@ -526,10 +558,10 @@ const char* AVSC_CC avs_prop_get_data(AVS_ScriptEnvironment * p, const AVS_Map *
   p->error = 0;
   try {
     const char* data = p->env->propGetData((const AVSMap*)map, key, index, error);
-    if (error)
+    if (!data)
       return nullptr;
     else
-      return p->env->SaveString(data);
+      return data;
   }
   catch (const AvisynthError& err) {
     p->error = err.msg;
@@ -729,6 +761,8 @@ PVideoFrame C_VideoFilter::GetFrame(int n, IScriptEnvironment* env)
     AVS_VideoFrame* f = d.get_frame(&d, n);
     if (d.error)
       throw AvisynthError(d.error);
+    if (d.child != NULL && d.child->error)
+      throw AvisynthError(d.child->error);
     PVideoFrame fr((VideoFrame*)f);
     ((PVideoFrame*)&f)->~PVideoFrame();
     return fr;
@@ -1130,12 +1164,11 @@ AVS_VideoFrame * AVSC_CC avs_new_video_frame_a(AVS_ScriptEnvironment * p, const
 // with frame properties, and alignment
 // note: in general there is no need for alignment specificationm use avs_new_video_frame_p
 extern "C"
-AVS_VideoFrame * AVSC_CC avs_new_video_frame_p_a(AVS_ScriptEnvironment * p, const AVS_VideoInfo * vi, AVS_VideoFrame * propSrc, int align)
+AVS_VideoFrame * AVSC_CC avs_new_video_frame_p_a(AVS_ScriptEnvironment * p, const AVS_VideoInfo * vi, const AVS_VideoFrame * prop_src, int align)
 {
   p->error = 0;
   try {
-    auto env = p->env;
-    PVideoFrame f0 = env->NewVideoFrameP(*(const VideoInfo*)vi, (PVideoFrame*)propSrc, align);
+    PVideoFrame f0 = p->env->NewVideoFrameP(*(const VideoInfo*)vi, (const PVideoFrame*)&prop_src, align);
     AVS_VideoFrame* f;
     new((PVideoFrame*)&f) PVideoFrame(f0);
     return f;
@@ -1148,12 +1181,11 @@ AVS_VideoFrame * AVSC_CC avs_new_video_frame_p_a(AVS_ScriptEnvironment * p, cons
 
 // with frame properties, no extra alignment requirement
 extern "C"
-AVS_VideoFrame * AVSC_CC avs_new_video_frame_p(AVS_ScriptEnvironment * p, const AVS_VideoInfo * vi, AVS_VideoFrame * propSrc)
+AVS_VideoFrame * AVSC_CC avs_new_video_frame_p(AVS_ScriptEnvironment * p, const AVS_VideoInfo * vi, const AVS_VideoFrame * prop_src)
 {
   p->error = 0;
   try {
-    auto env = p->env;
-    PVideoFrame f0 = env->NewVideoFrameP(*(const VideoInfo*)vi, (PVideoFrame*)propSrc, AVS_FRAME_ALIGN);
+    PVideoFrame f0 = p->env->NewVideoFrameP(*(const VideoInfo*)vi, (const PVideoFrame*)&prop_src, AVS_FRAME_ALIGN);
     AVS_VideoFrame* f;
     new((PVideoFrame*)&f) PVideoFrame(f0);
     return f;
