@@ -243,7 +243,7 @@ int VideoInfo::GetPlaneHeightSubsampling(int plane) const {  // Subsampling in b
     else
       throw AvisynthError("Filter error: GetPlaneHeightSubsampling called with unsupported pixel type.");
   }
-  throw AvisynthError("Filter error: GetPlaneHeightSubsampling called with supported plane.");
+  throw AvisynthError("Filter error: GetPlaneHeightSubsampling called with unsupported plane.");
 }
 
 int VideoInfo::BitsPerPixel() const {
@@ -402,6 +402,33 @@ bool VideoInfo::IsRGB64()   const { return ((pixel_type & CS_BGR32) == CS_BGR32)
 bool VideoInfo::IsYUVA() const { return !!(pixel_type&CS_YUVA ); }
 bool VideoInfo::IsPlanarRGB() const { return !!(pixel_type&CS_PLANAR) && !!(pixel_type&CS_BGR) && !!(pixel_type&CS_RGB_TYPE); }
 bool VideoInfo::IsPlanarRGBA() const { return !!(pixel_type&CS_PLANAR) && !!(pixel_type&CS_BGR) && !!(pixel_type&CS_RGBA_TYPE); }
+bool VideoInfo::IsChannelMaskKnown() const { return !!(image_type & IT_HAS_CHANNELMASK); }
+// Re-maps and stores channel mask into image_type, sets the 'has channel mask' flag as well
+void VideoInfo::SetChannelMask(bool isChannelMaskKnown, unsigned int dwChannelMask)
+{
+  if (isChannelMaskKnown) image_type |= IT_HAS_CHANNELMASK; else image_type &= ~IT_HAS_CHANNELMASK;
+  if (!isChannelMaskKnown) dwChannelMask = 0;
+  if (dwChannelMask & AvsChannelMask::MASK_SPEAKER_ALL) // special mapping due to lack of bits in image_type
+    dwChannelMask = AvsImageTypeFlags::IT_SPEAKER_ALL;
+  else {
+    dwChannelMask &= AvsChannelMask::MASK_SPEAKER_DEFINED;
+    dwChannelMask <<= 4;
+  }
+  image_type &= ~AvsImageTypeFlags::IT_SPEAKER_BITS_MASK;
+  image_type |= dwChannelMask;
+}
+
+unsigned int VideoInfo::GetChannelMask() const
+{
+  // returns zero if not defined
+  if (!IsChannelMaskKnown()) return 0;
+  unsigned int dwChannelMask = image_type & AvsImageTypeFlags::IT_SPEAKER_BITS_MASK;
+  // returns the original SPEAKER_ALL constant
+  if (dwChannelMask == AvsImageTypeFlags::IT_SPEAKER_ALL)
+    return AvsChannelMask::MASK_SPEAKER_ALL;
+  // other SPEAKER bits were simply shifted by 4 bits
+  return dwChannelMask >> 4;
+}
 
 
 // end struct VideoInfo
@@ -478,8 +505,10 @@ void VideoFrame::Release() {
   VideoFrameBuffer* _vfb = vfb;
 
   if (!InterlockedDecrement(&refcount)) {
-    if (properties)
-      properties->clear();
+    if (properties != nullptr) {
+      delete properties; // if needed, frame registry will re-create
+      properties = nullptr;
+    }
     InterlockedDecrement(&_vfb->refcount);
   }
 }
@@ -527,6 +556,9 @@ int VideoFrame::GetHeight(int plane) const {
   return height;
 }
 
+int VideoFrame::GetPixelType() const { return pixel_type; }
+void VideoFrame::AmendPixelType(int new_pixel_type) { pixel_type = new_pixel_type; }
+
 // Generally you should not be using these two
 VideoFrameBuffer* VideoFrame::GetFrameBuffer() const { return vfb; }
 int VideoFrame::GetOffset(int plane) const {
@@ -601,7 +633,14 @@ int VideoFrame::CheckMemory() const {
 VideoFrame::~VideoFrame() { InterlockedDecrement(&vfb->refcount); }
    Baked ********************/
 VideoFrame::~VideoFrame()     { DESTRUCTOR(); }
-void VideoFrame::DESTRUCTOR() { Release(); }
+void VideoFrame::DESTRUCTOR() { 
+  Release(); 
+  // finally destroyed when removed from FrameRegistry
+  if (properties) {
+    delete properties;
+    properties = nullptr;
+  }
+}
 
 // end class VideoFrame
 
@@ -912,6 +951,8 @@ void AVSValue::Assign2(const AVSValue* src, bool init, bool c_arrays) {
     ((IFunction *)prev_pointer_to_release)->Release();
 }
 
+AvsValueType AVSValue::GetType() const { return (AvsValueType)type; }
+
 // end class AVSValue
 
 /**********************************************************************/
@@ -968,7 +1009,7 @@ PNeoEnv::operator IScriptEnvironment2* () { return static_cast<InternalEnvironme
 
 /**********************************************************************/
 
-static const AVS_Linkage avs_linkage = {   // struct AVS_Linkage {
+static const AVS_Linkage avs_linkage = {    // struct AVS_Linkage {
 
   sizeof(AVS_Linkage),                      //   int Size;
 
@@ -1157,11 +1198,11 @@ static const AVS_Linkage avs_linkage = {   // struct AVS_Linkage {
   &PFunction::DESTRUCTOR,
   // end PFunction
 
-  // Videoframe extras (Neo)
+  // VideoFrame extras (Neo)
   &VideoFrame::CheckMemory,
   &VideoFrame::GetDevice,
 
-    // class PDevice (Neo)
+  // class PDevice (Neo)
   &PDevice::CONSTRUCTOR0,
   &PDevice::CONSTRUCTOR1,
   &PDevice::CONSTRUCTOR2,
@@ -1174,15 +1215,20 @@ static const AVS_Linkage avs_linkage = {   // struct AVS_Linkage {
   &PDevice::GetName,
   // end class PDevice
 
-  &VideoFrame::IsPropertyWritable,          //   V9
+  // V9
+  &VideoFrame::IsPropertyWritable,
+
+  // V10
+  &VideoFrame::GetPixelType,                //   int          (VideoFrame::*VideoFrame_GetPixelType)() const;
+  &VideoFrame::AmendPixelType,              //   void         (VideoFrame::*VideoFrame_AmendPixelType)();
+  &VideoFrameBuffer::DESTRUCTOR,            //   void         (VideoFrameBuffer::*VideoFrameBuffer_DESTRUCTOR)();
+  &AVSValue::GetType,                       //   AvsValueType (AVSValue::*AVSValue_GetType)() const;
+
+  // V10.1
+  &VideoInfo::IsChannelMaskKnown,           //   bool    (VideoInfo::*IsChannelMaskKnown)()  const;
+  &VideoInfo::SetChannelMask,               //   void    (VideoInfo::*SetChannelMask)();
+  &VideoInfo::GetChannelMask,               //   int     (VideoInfo::*GetChannelMask)()  const;
 
-  NULL,                                     //   reserved for AviSynth+
-  NULL,                                     //   reserved for AviSynth+
-  NULL,                                     //   reserved for AviSynth+
-  NULL,                                     //   reserved for AviSynth+
-  NULL,                                     //   reserved for AviSynth+
-  NULL,                                     //   reserved for AviSynth+
-  NULL,                                     //   reserved for AviSynth+
   NULL,                                     //   reserved for AviSynth+
   NULL,                                     //   reserved for AviSynth+
   NULL,                                     //   reserved for AviSynth+
