@@ -55,9 +55,17 @@
 #endif
 #endif
 
+#if AVS_DEVNEXT_REV > 0
+#define AVS_DEVELOPMENT_BUILD "\nNext Version Development Build: " AVS_PPSTR(AVS_DEV_REVDATE)
+#define AVS_DEVELOPMENT_BUILD_GIT "\nGit: " AVS_PPSTR(AVS_DEV_GITHASH)
+#else
+#define AVS_DEVELOPMENT_BUILD
+#define AVS_DEVELOPMENT_BUILD_GIT
+#endif
+
 #define AVS_CLASSIC_VERSION 2.60  // Note: Used by VersionNumber() script function
-#define AVS_COPYRIGHT "\n\xA9 2000-2015 Ben Rudiak-Gould, et al.\nhttp://avisynth.nl\n\xA9 2013-2022 AviSynth+ Project"
-#define AVS_COPYRIGHT_UTF8 u8"\n\u00A9 2000-2015 Ben Rudiak-Gould, et al.\nhttp://avisynth.nl\n\u00A9 2013-2022 AviSynth+ Project"
+#define AVS_COPYRIGHT "\n\xA9 2000-2015 Ben Rudiak-Gould, et al.\nhttp://avisynth.nl\n\xA9 2013-2023 AviSynth+ Project"
+#define AVS_COPYRIGHT_UTF8 u8"\n\u00A9 2000-2015 Ben Rudiak-Gould, et al.\nhttp://avisynth.nl\n\u00A9 2013-2023 AviSynth+ Project"
 #define BUILTIN_FUNC_PREFIX "AviSynth"
 
 enum MANAGE_CACHE_KEYS
