@@ -53,14 +53,14 @@
 #include <cstdio>
 #include <new>
 #include "AviHelper.h"
-
+#include "audio.h"
 
 #include <float.h>
 
 #ifdef MSVC
 constexpr uint32_t FP_STATE = 0x9001f;
 #ifdef _M_X64
-constexpr uint32_t FP_MASK = 0xffffffff & !(_MCW_PC || _MCW_IC);
+constexpr uint32_t FP_MASK = 0xffffffff & !(_MCW_PC | _MCW_IC);
 // x64 ignores _MCW_PC and _MCW_IC, Debug CRT library actually asserts when these are passed.
 // https ://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/control87-controlfp-control87-2?redirectedfrom=MSDN&view=msvc-170
 #else
@@ -146,8 +146,9 @@ struct IAvisynthClipInfo : IUnknown {
   virtual bool __stdcall IsFieldBased() = 0;
 };
 
-
-class CAVIFileSynth: public IAVIFile, public IPersistFile, public IClassFactory, public IAvisynthClipInfo {
+// final is used to silence warning:
+// delete called on non-final 'CAVIFileSynth' that has virtual functions but non-virtual destructor [-Wdelete-non-abstract-non-virtual-dtor]
+class CAVIFileSynth final: public IAVIFile, public IPersistFile, public IClassFactory, public IAvisynthClipInfo {
   friend class CAVIStreamSynth;
 private:
   long m_refs;
@@ -189,6 +190,10 @@ public:
 
   //////////// IUnknown
 
+  // 2023: Lots of clang-cl LLVM warning:
+  // "exception specification of overriding function is more lax than base version[-Wmicrosoft-exception-spec]"
+  // it's because STDMETHOD and STDMETHODIMP is differing by COM_DECLSPEC_NOTHROW that is __declspec(nothrow).
+  // Cannot do anything against that, this is by MS design
   STDMETHODIMP QueryInterface(const IID& iid, void** ppv);
   STDMETHODIMP_(ULONG) AddRef();
   STDMETHODIMP_(ULONG) Release();
@@ -234,7 +239,9 @@ public:
 
 class CAVIStreamSynth;
 
-class CAVIStreamSynth: public IAVIStream, public IAVIStreaming {
+// final is used to silence warning:
+// warning : delete called on non-final 'CAVIStreamSynth' that has virtual functions but non-virtual destructor
+class CAVIStreamSynth final: public IAVIStream, public IAVIStreaming {
 public:
 
   //////////// IUnknown
@@ -1541,7 +1548,10 @@ STDMETHODIMP CAVIStreamSynth::ReadFormat(LONG lPos, LPVOID lpFormat, LONG *lpcbF
 
   if (!lpcbFormat) return E_POINTER;
 
-  const bool UseWaveExtensible = parent->env->GetVarBool(VARNAME_UseWaveExtensible, false);
+  const VideoInfo* const vi = parent->vi;
+
+  const bool UseWaveExtDefault = vi->IsChannelMaskKnown();
+  const bool UseWaveExtensible = parent->env->GetVarBool(VARNAME_UseWaveExtensible, UseWaveExtDefault);
 
   if (!lpFormat) {
     *lpcbFormat = fAudio ? ( UseWaveExtensible ? sizeof(WAVEFORMATEXTENSIBLE)
@@ -1552,8 +1562,6 @@ STDMETHODIMP CAVIStreamSynth::ReadFormat(LONG lPos, LPVOID lpFormat, LONG *lpcbF
 
   memset(lpFormat, 0, *lpcbFormat);
 
-  const VideoInfo* const vi = parent->vi;
-
   if (fAudio) {
     if (UseWaveExtensible) {  // Use WAVE_FORMAT_EXTENSIBLE audio output format
 #ifndef MSVC
@@ -1574,23 +1582,18 @@ STDMETHODIMP CAVIStreamSynth::ReadFormat(LONG lPos, LPVOID lpFormat, LONG *lpcbF
       wfxt.Format.cbSize = sizeof(wfxt) - sizeof(wfxt.Format);
       wfxt.Samples.wValidBitsPerSample = wfxt.Format.wBitsPerSample;
 
-      const int SpeakerMasks[9] = { 0,
-        0x00004, // 1   -- -- Cf
-        0x00003, // 2   Lf Rf
-        0x00007, // 3   Lf Rf Cf
-        0x00033, // 4   Lf Rf -- -- Lr Rr
-        0x00037, // 5   Lf Rf Cf -- Lr Rr
-        0x0003F, // 5.1 Lf Rf Cf Sw Lr Rr
-        0x0013F, // 6.1 Lf Rf Cf Sw Lr Rr -- -- Cr
-        0x0063F, // 7.1 Lf Rf Cf Sw Lr Rr -- -- -- Ls Rs
-      };
-      wfxt.dwChannelMask = (unsigned)vi->AudioChannels() <= 8 ? SpeakerMasks[vi->AudioChannels()]
-        : (unsigned)vi->AudioChannels() <=18 ? DWORD(-1) >> (32-vi->AudioChannels())
-        : SPEAKER_ALL;
+      // If is set, then let's use it.
+      if (vi->IsChannelMaskKnown())
+        wfxt.dwChannelMask = vi->GetChannelMask();
+      else {
+        wfxt.dwChannelMask = (unsigned)vi->AudioChannels() <= 8 ? GetDefaultChannelLayout(vi->AudioChannels())
+          : (unsigned)vi->AudioChannels() <= 18 ? DWORD(-1) >> (32 - vi->AudioChannels())
+          : SPEAKER_ALL;
 
-      unsigned int userChannelMask = (unsigned)(parent->env->GetVarInt(VARNAME_dwChannelMask, 0));
-      if (userChannelMask != 0)
-        wfxt.dwChannelMask = userChannelMask;
+        unsigned int userChannelMask = (unsigned)(parent->env->GetVarInt(VARNAME_dwChannelMask, 0));
+        if (userChannelMask != 0)
+          wfxt.dwChannelMask = userChannelMask;
+      }
 
       wfxt.SubFormat = vi->IsSampleType(SAMPLE_FLOAT) ? KSDATAFORMAT_SUBTYPE_IEEE_FLOAT : KSDATAFORMAT_SUBTYPE_PCM;
       *lpcbFormat = min(*lpcbFormat, (LONG)sizeof(wfxt));
