@@ -581,6 +581,7 @@ AVSValue MinMaxPlane::Create_min(AVSValue args, void* user_data, IScriptEnvironm
 
 AVSValue MinMaxPlane::Create_median(AVSValue args, void* user_data, IScriptEnvironment* env) {
   int plane = (int)reinterpret_cast<intptr_t>(user_data);
+  // MEDIAN: min 50%
   return MinMax(args[0], user_data, 50.0, args[1].AsInt(0), plane, MinMaxPlane::MIN, false, env);
 }
 
@@ -589,14 +590,18 @@ AVSValue MinMaxPlane::Create_minmax(AVSValue args, void* user_data, IScriptEnvir
   return MinMax(args[0], user_data, args[1].AsDblDef(0.0), args[2].AsInt(0), plane, MinMaxPlane::MINMAX_DIFFERENCE, false, env);
 }
 
-void get_minmax_float_c(const BYTE* srcp, int pitch, int w, int h, float& min, float& max)
+template<bool average>
+void get_minmax_float_c(const BYTE* srcp, int pitch, int w, int h, float& min, float& max, double &sum)
 {
   min = *reinterpret_cast<const float*>(srcp);
   max = min;
+  sum = 0;
 
   for (int y = 0; y < h; y++) {
     for (int x = 0; x < w; x++) {
       const float pix = reinterpret_cast<const float*>(srcp)[x];
+      if constexpr(average)
+        sum += pix;
       if (pix < min) min = pix;
       if (pix > max) max = pix;
     }
@@ -604,18 +609,24 @@ void get_minmax_float_c(const BYTE* srcp, int pitch, int w, int h, float& min, f
   }
 }
 
-template<typename pixel_t>
-void get_minmax_int_c(const BYTE* srcp, int pitch, int w, int h, int& min, int& max)
+template<typename pixel_t, bool average>
+void get_minmax_int_c(const BYTE* srcp, int pitch, int w, int h, int& min, int& max, int64_t& sum)
 {
   min = *reinterpret_cast<const pixel_t*>(srcp);
   max = min;
+  sum = 0;
 
   for (int y = 0; y < h; y++) {
+    int tmpsum = 0;
     for (int x = 0; x < w; x++) {
       const int pix = reinterpret_cast<const pixel_t*>(srcp)[x];
+      if constexpr (average)
+        sum += pix;
       if (pix < min) min = pix;
       if (pix > max) max = pix;
     }
+    if constexpr (average)
+      sum += tmpsum;
     srcp += pitch;
   }
 }
@@ -686,27 +697,40 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
   float stats_median;
   float stats_thresholded_min;
   float stats_thresholded_max;
+  float stats_average;
 
   if (threshold == 0 || mode == MinMaxPlane::STATS) {
     // special case, no histogram needed
 
     if (pixelsize == 4) // 32 bit float
     {
-      get_minmax_float_c(srcp, pitch, w, h, stats_min, stats_max);
-      
+      double sum = 0;
+      if (mode == MinMaxPlane::STATS)
+        get_minmax_float_c<true>(srcp, pitch, w, h, stats_min, stats_max, sum);
+      else
+        get_minmax_float_c<false>(srcp, pitch, w, h, stats_min, stats_max, sum);
+
       if (mode == MinMaxPlane::MIN) return stats_min;
       else if (mode == MinMaxPlane::MAX) return stats_max;
       else if (mode == MinMaxPlane::MINMAX_DIFFERENCE) return stats_max - stats_min;
+      stats_average = (float)(sum / (w * h));
       // STATS: go on
     }
     else
     {
       int min, max;
+      int64_t sum = 0;
       // sse4.1 is not any faster
       if(pixelsize == 1)
-        get_minmax_int_c<uint8_t>(srcp, pitch, w, h, min, max);
+        if (mode == MinMaxPlane::STATS)
+          get_minmax_int_c<uint8_t, true>(srcp, pitch, w, h, min, max, sum);
+        else
+          get_minmax_int_c<uint8_t, false>(srcp, pitch, w, h, min, max, sum);
       else
-        get_minmax_int_c<uint16_t>(srcp, pitch, w, h, min, max);
+        if (mode == MinMaxPlane::STATS)
+          get_minmax_int_c<uint16_t, true>(srcp, pitch, w, h, min, max, sum);
+        else
+          get_minmax_int_c<uint16_t, false>(srcp, pitch, w, h, min, max, sum);
 
       if (mode == MinMaxPlane::MIN) return min;
       else if (mode == MinMaxPlane::MAX) return max;
@@ -715,6 +739,7 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
       // STATS: go on
       stats_min = (float)min;
       stats_max = (float)max;
+      stats_average = (float)((double)sum / (w * h));
     }
   }
 
@@ -781,6 +806,7 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
 
     // Find the value we need.
   if (mode == MinMaxPlane::MIN) {
+    // threshold == 0 was covered already
     unsigned int counted=0;
     retval = buffersize - 1;
     for (int i = 0; i< buffersize;i++) {
@@ -791,8 +817,7 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
       }
     }
   }
-  
-  if (mode == MinMaxPlane::MAX) {
+  else if (mode == MinMaxPlane::MAX) {
     unsigned int counted=0;
     retval = 0;
     for (int i = buffersize-1; i>=0;i--) {
@@ -803,8 +828,13 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
       }
     }
   }
-  
-  if (mode == MinMaxPlane::MINMAX_DIFFERENCE || mode == MinMaxPlane::STATS) {
+  else if (mode == MinMaxPlane::STATS && threshold == 0) {
+    stats_thresholded_min = (float)stats_min;
+    stats_thresholded_max = (float)stats_max;
+    retval = -1; // stats: n/a
+  }
+  else if (mode == MinMaxPlane::MINMAX_DIFFERENCE || mode == MinMaxPlane::STATS) {
+    // thresholded case
     unsigned int counted=0;
     int i, t_min = 0;
     // Find min
@@ -835,6 +865,7 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
   if (mode == MinMaxPlane::STATS) {
     // for STATS we already have min and max, thresholded min and max
     // let's gather median, which is equal to 50% MIN
+    // comment: no separate MEDIAN mode is used, already converted to MIN 50% in ctor
     unsigned int tpixels = (unsigned int)(pixels * 0.5f);
     unsigned int counted = 0;
     retval = buffersize - 1;
@@ -847,30 +878,28 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
     }
     stats_median = (float)retval;
   }
-  else {
-    retval = -1;
-  }
 
   delete[] accum_buf;
   //_RPT2(0, "End of MinMax cn=%d n=%d\r", cn.AsInt(), n);
 
   if (mode == MinMaxPlane::STATS) {
-  if (pixelsize == 4) {
-    const bool chroma = (plane == PLANAR_U) || (plane == PLANAR_V);
+    if (pixelsize == 4) {
+      const bool chroma = (plane == PLANAR_U) || (plane == PLANAR_V);
       const float shift = chroma ? 32768.0f : 0;
       stats_thresholded_min = (float)((double)(stats_thresholded_min - shift) / (buffersize - 1)); // convert back to float, /65535
       stats_thresholded_max = (float)((double)(stats_thresholded_max - shift) / (buffersize - 1)); // convert back to float, /65535
       stats_median = (float)((double)(stats_median - shift) / (buffersize - 1)); // convert back to float, /65535
     }
 
-    AVSValue result[] = { stats_min, stats_max, stats_thresholded_min, stats_thresholded_max, stats_median };
-    AVSValue ret = AVSValue(result, 5);
+    AVSValue result[] = { stats_min, stats_max, stats_thresholded_min, stats_thresholded_max, stats_median, stats_average };
+    AVSValue ret = AVSValue(result, 6);
     if (setvar) {
       env->SetGlobalVar("PlaneStats_min", stats_min);
       env->SetGlobalVar("PlaneStats_max", stats_max);
       env->SetGlobalVar("PlaneStats_thmin", stats_thresholded_min);
       env->SetGlobalVar("PlaneStats_thmax", stats_thresholded_max);
       env->SetGlobalVar("PlaneStats_median", stats_median);
+      env->SetGlobalVar("PlaneStats_average", stats_average);
     }
 
     return ret;
