@@ -663,7 +663,18 @@ SeparateFields::SeparateFields(PClip _child, IScriptEnvironment* env)
 
 PVideoFrame SeparateFields::GetFrame(int n, IScriptEnvironment* env)
 {
+#ifdef CACHE_GROWTH_INFINITELY_TEST
+  // FIXME: debug for Issue #270
+  // See other occurencies of this define
+  // When filter is combined with non-SeparateFielded frames
+  // the cache can grow infinitely, behaves like a memory leak.
+  // Tried putting n or (n-1) or (n*2) instead of n >> 1 then the problem does not occur.
+  _RPT2(0, "SeparateFields::GetFrame before %d, >>1: %d\n", n, n >> 1);
+#endif
   PVideoFrame frame = child->GetFrame(n>>1, env);
+#ifdef CACHE_GROWTH_INFINITELY_TEST
+  _RPT2(0, "SeparateFields::GetFrame after %d, >>1: %d\n", n, n >> 1);
+#endif
   if (vi.IsPlanar()) {
     const bool topfield = GetParity(n);
 
@@ -788,7 +799,7 @@ AVSValue __cdecl SelectEvery::Create(AVSValue args, void*, IScriptEnvironment* e
 {
   const int num_vals = args[2].ArraySize();
   if (num_vals <= 1)
-	return new SelectEvery(args[0].AsClip(), args[1].AsInt(), num_vals>0 ? args[2][0].AsInt() : 0, env);
+  return new SelectEvery(args[0].AsClip(), args[1].AsInt(), num_vals>0 ? args[2][0].AsInt() : 0, env);
   else {
     PClip* child_array = new PClip[num_vals];
     for (int i=0; i<num_vals; ++i)
@@ -1004,7 +1015,7 @@ static AVSValue __cdecl Create_Pulldown(AVSValue args, void*, IScriptEnvironment
 static AVSValue __cdecl Create_SwapFields(AVSValue args, void*, IScriptEnvironment* env)
 {
   return new SelectEvery(new DoubleWeaveFields(new ComplementParity(
-	  new SeparateFields(args[0].AsClip(), env))), 2, 0, env);
+    new SeparateFields(args[0].AsClip(), env))), 2, 0, env);
 }
 
 
@@ -1066,7 +1077,7 @@ bool __stdcall SelectRangeEvery::GetParity(int n)
 void __stdcall SelectRangeEvery::GetAudio(void* buf, int64_t start, int64_t count, IScriptEnvironment* env)
 {
   if (!audio) {
-	// Use original unTrim'd child
+  // Use original unTrim'd child
     achild->GetAudio(buf, start, count, env);
     return;
   }
