@@ -58,8 +58,8 @@
 #include <stdint.h>
 
 
-#define PI        3.141592653589793
-
+constexpr double PI = 3.14159265358979323846; 
+// until c++20 <numbers> std::numbers::pi
 
 /********************************************************************
 ***** Declare index of new filters for Avisynth's filter engine *****
@@ -167,8 +167,13 @@ Histogram::Histogram(PClip _child, Mode _mode, AVSValue _option, int _show_bits,
       vi.width = (1 << show_bits); // 256 for 8 bit
       vi.height = (1 << show_bits); // yes, height can change
     }
-    int half = 1 << (show_bits - 1); // 127
-    int R = half - 1; // 126
+
+    // precalculate 15 degree marker dots
+    const int half = (1 << (show_bits - 1)) - 1; // 127
+    // dots are placed somewhat inside to the colorful circle, which is thicker for higher show_bits
+    color2_innerF = 124.9;  // .9 is for better visuals in subsampled mode
+    int R = (int)(1 + color2_innerF * (1 << (show_bits - 8)) + 0.5); // 126 for 8 bits
+
     for (int y=0; y<24; y++) { // just inside the big circle
       deg15c[y] = (int) ( R*cos(y*PI/12.) + 0.5) + half;
       deg15s[y] = (int) (-R*sin(y*PI/12.) + 0.5) + half;
@@ -592,25 +597,27 @@ PVideoFrame Histogram::DrawModeLuma(int n, IScriptEnvironment* env) {
       auto dstp_u = src->GetWritePtr(PLANAR_U);
       auto dstp_v = src->GetWritePtr(PLANAR_V);
       auto height_uv = src->GetHeight(PLANAR_U);
+      auto rowsize_uv = src->GetRowSize(PLANAR_U);
       auto pitch_uv = src->GetPitch(PLANAR_U);
       if (bits_per_pixel == 8)
-        fill_chroma<uint8_t>(dstp_u, dstp_v, height_uv, pitch_uv, 128);
+        fill_chroma<uint8_t>(dstp_u, dstp_v, height_uv, rowsize_uv, pitch_uv, 128);
       else if (bits_per_pixel <= 16)
-        fill_chroma<uint16_t>(dstp_u, dstp_v, height_uv, pitch_uv, 128 << (bits_per_pixel - 8));
+        fill_chroma<uint16_t>(dstp_u, dstp_v, height_uv, rowsize_uv, pitch_uv, 128 << (bits_per_pixel - 8));
       else // 32)
-        fill_chroma<float>(dstp_u, dstp_v, height_uv, pitch_uv, 0.0f);
+        fill_chroma<float>(dstp_u, dstp_v, height_uv, rowsize_uv, pitch_uv, 0.0f);
 
       // alpha
       if (vi.NumComponents() == 4) {
         auto dstp_a = src->GetWritePtr(PLANAR_A);
         auto height_a = src->GetHeight(PLANAR_A);
+        auto rowsize_a = src->GetRowSize(PLANAR_A);
         auto pitch_a = src->GetPitch(PLANAR_A);
         if (bits_per_pixel == 8)
-          fill_plane<uint8_t>(dstp_a, height_a, pitch_a, 255);
+          fill_plane<uint8_t>(dstp_a, height_a, rowsize_a, pitch_a, 255);
         else if (bits_per_pixel <= 16)
-          fill_plane<uint16_t>(dstp_a, height_a, pitch_a, (1 << bits_per_pixel) -  1);
+          fill_plane<uint16_t>(dstp_a, height_a, rowsize_a, pitch_a, (1 << bits_per_pixel) -  1);
         else // 32)
-          fill_chroma<float>(dstp_u, dstp_v, height_uv, pitch_uv, 1.0f);
+          fill_plane<float>(dstp_a, height_a, rowsize_a, pitch_a, 1.0f);
       }
     }
   }
@@ -648,15 +655,25 @@ void DrawModeColor2_erase_area(int bits_per_pixel,
 
 template<typename pixel_t>
 void DrawModeColor2_draw_misc(int bits_per_pixel,
-  uint8_t* dstp, uint8_t* dstp_u, uint8_t* dstp_v,
+  uint8_t* dstp8, uint8_t* dstp8_u, uint8_t* dstp8_v,
   int pitch, int pitchUV,
   int height, int heightUV,
-  int show_bits, int swidth, int sheight)
+  int show_bits, int swidth, int sheight,
+  double innerF
+)
 {
   pixel_t black;
   pixel_t middle_chroma;
   pixel_t luma128;
 
+  pitch /= sizeof(pixel_t);
+  pitchUV /= sizeof(pixel_t);
+  pixel_t* dstp = reinterpret_cast<pixel_t*>(dstp8);
+  pixel_t* dstp_u = reinterpret_cast<pixel_t*>(dstp8_u);
+  pixel_t* dstp_v = reinterpret_cast<pixel_t*>(dstp8_v);
+
+
+
   if constexpr (std::is_integral<pixel_t>::value) {
     black = 16 << (bits_per_pixel - 8);
     middle_chroma = (pixel_t)(128 << (bits_per_pixel - 8));
@@ -673,30 +690,37 @@ void DrawModeColor2_draw_misc(int bits_per_pixel,
 
   // Erase all - luma
   for (int y = 0; y < height; y++) {
-    uint8_t* ptr = dstp + y * pitch;
-    std::fill_n((pixel_t *)ptr, showsize, black);
+    pixel_t* ptr = dstp + y * pitch;
+    std::fill_n(ptr, showsize, black);
   }
 
   // Erase all - chroma
   for (int y = 0; y < heightUV; y++) {
-    uint8_t* ptrU = dstp_u + y * pitchUV;
-    uint8_t* ptrV = dstp_v + y * pitchUV;
-    std::fill_n((pixel_t*)ptrU, showsizeUV, middle_chroma);
-    std::fill_n((pixel_t*)ptrV, showsizeUV, middle_chroma);
+    pixel_t* ptrU = dstp_u + y * pitchUV;
+    pixel_t* ptrV = dstp_v + y * pitchUV;
+    std::fill_n(ptrU, showsizeUV, middle_chroma);
+    std::fill_n(ptrV, showsizeUV, middle_chroma);
   }
 
-  // possible to display 10 bit data stuffed into 8 bit size
+  // possible to display >8 bit data stuffed into 8 bit size
   const int show_bit_shift = show_bits - 8;
 
+  //    16       240
+  // 16  +---------+
+  // 17  |         |
+  // ..  |         |
+  // 239 |         |
+  // 240 +---------+
+
   // plot valid grey ccir601 square
-  const int size = (240 - 16 + 1) << show_bit_shift; // original 8 bit: 225 
-  std::fill_n((pixel_t*)(&dstp[((16 << show_bit_shift) * pitch) + (16 << show_bit_shift) * sizeof(pixel_t)]), size, luma128);
-  std::fill_n((pixel_t*)(&dstp[((240 << show_bit_shift) * pitch) + (16 << show_bit_shift) * sizeof(pixel_t)]), size, luma128);
+  const int size = 1 + ((240 - 16) << show_bit_shift); // original 8 bit: 225 
+  std::fill_n(&dstp[(16 << show_bit_shift) * pitch + (16 << show_bit_shift)], size, luma128);
+  std::fill_n(&dstp[(240 << show_bit_shift) * pitch + (16 << show_bit_shift)], size, luma128);
 
   // vertical lines left and right side
-  for (int y = 17 << show_bit_shift; y < 240 << show_bit_shift; y++) {
-    ((pixel_t*)dstp)[(16 << show_bit_shift) + y * pitch / sizeof(pixel_t)] = luma128;
-    ((pixel_t*)dstp)[(240 << show_bit_shift) + y * pitch / sizeof(pixel_t)] = luma128;
+  for (int y = 1 + (16 << show_bit_shift); y < 240 << show_bit_shift; y++) {
+    dstp[(16 << show_bit_shift) + y * pitch] = luma128;
+    dstp[(240 << show_bit_shift) + y * pitch] = luma128;
   }
 
   // plot circles
@@ -753,7 +777,7 @@ void DrawModeColor2_draw_misc(int bits_per_pixel,
   // similar for the other hues.
   // luma
 
-  double innerF = 124.9;  // .9 is for better visuals in subsampled mode
+  // innerF is used for 15 degree markers as well
   double thicknessF = 1.5;
   double oneOverThicknessF = 1.0 / thicknessF;
   double outerF = innerF + thicknessF * 2.0;
@@ -783,14 +807,10 @@ void DrawModeColor2_draw_misc(int bits_per_pixel,
           int xP = limit + x;
           int yP = limit + y;
 
-          pixel_t* pdstb = (pixel_t*)dstp;
-          pixel_t* pdstbU = (pixel_t*)dstp_u;
-          pixel_t* pdstbV = (pixel_t*)dstp_v;
-
-          pdstb[xP + yP * pitch / sizeof(pixel_t)] =
+          dstp[xP + yP * pitch] =
             (pixel_t)((interp * (LC[3 * activeY] << factorshift)) >> 8); // left upper half
 
-          pdstb[limit_showwidth - xP + yP * pitch / sizeof(pixel_t)] =
+          dstp[limit_showwidth - xP + yP * pitch] =
             (pixel_t)((interp * (RC[3 * activeY] << factorshift)) >> 8); // right upper half
 
           xP = (xP + xRounder) >> swidth;
@@ -800,15 +820,15 @@ void DrawModeColor2_draw_misc(int bits_per_pixel,
           int invInt = (MAXINTERP - interp);
           
           int p_uv;
-          p_uv = xP + yP * pitchUV / sizeof(pixel_t);
-          pdstbU[p_uv] = (pixel_t)((pdstbU[p_uv] * invInt + interp * (LC[3 * activeY + 1] << factorshift)) >> 8); // left half
-          pdstbV[p_uv] = (pixel_t)((pdstbV[p_uv] * invInt + interp * (LC[3 * activeY + 2] << factorshift)) >> 8); // left half
+          p_uv = xP + yP * pitchUV;
+          dstp_u[p_uv] = (pixel_t)((dstp_u[p_uv] * invInt + interp * (LC[3 * activeY + 1] << factorshift)) >> 8); // left half
+          dstp_v[p_uv] = (pixel_t)((dstp_v[p_uv] * invInt + interp * (LC[3 * activeY + 2] << factorshift)) >> 8); // left half
 
           xP = ((limit_showwidth) >> swidth) - xP;
-          p_uv = xP + yP * pitchUV / sizeof(pixel_t);
+          p_uv = xP + yP * pitchUV;
 
-          pdstbU[p_uv] = (pixel_t)((pdstbU[p_uv] * invInt + interp * (RC[3 * activeY + 1] << factorshift)) >> 8); // right half
-          pdstbV[p_uv] = (pixel_t)((pdstbV[p_uv] * invInt + interp * (RC[3 * activeY + 2] << factorshift)) >> 8); // right half
+          dstp_u[p_uv] = (pixel_t)((dstp_u[p_uv] * invInt + interp * (RC[3 * activeY + 1] << factorshift)) >> 8); // right half
+          dstp_v[p_uv] = (pixel_t)((dstp_v[p_uv] * invInt + interp * (RC[3 * activeY + 2] << factorshift)) >> 8); // right half
         }
         else {
           // 32 bit float
@@ -820,14 +840,10 @@ void DrawModeColor2_draw_misc(int bits_per_pixel,
           int xP = limit + x;
           int yP = limit + y;
 
-          pixel_t* pdstb = (pixel_t*)dstp;
-          pixel_t* pdstbU = (pixel_t*)dstp_u;
-          pixel_t* pdstbV = (pixel_t*)dstp_v;
-
-          pdstb[xP + yP * pitch / sizeof(pixel_t)] =
+          dstp[xP + yP * pitch] =
             (pixel_t)(interp * LC_f[3 * activeY]); // left upper half
 
-          pdstb[limit_showwidth - xP + yP * pitch / sizeof(pixel_t)] =
+          dstp[limit_showwidth - xP + yP * pitch] =
             (pixel_t)(interp * RC_f[3 * activeY]); // right upper half
 
           xP = (xP + xRounder) >> swidth;
@@ -837,15 +853,15 @@ void DrawModeColor2_draw_misc(int bits_per_pixel,
           float invInt = (MAXINTERP - interp);
 
           int p_uv;
-          p_uv = xP + yP * pitchUV / sizeof(pixel_t);
-          pdstbU[p_uv] = (pixel_t)(pdstbU[p_uv] * invInt + interp * LC_f[3 * activeY + 1]); // left half
-          pdstbV[p_uv] = (pixel_t)(pdstbV[p_uv] * invInt + interp * LC_f[3 * activeY + 2]); // left half
+          p_uv = xP + yP * pitchUV;
+          dstp_u[p_uv] = (pixel_t)(dstp_u[p_uv] * invInt + interp * LC_f[3 * activeY + 1]); // left half
+          dstp_v[p_uv] = (pixel_t)(dstp_v[p_uv] * invInt + interp * LC_f[3 * activeY + 2]); // left half
 
           xP = ((limit_showwidth) >> swidth) - xP;
 
-          p_uv = xP + yP * pitchUV / sizeof(pixel_t);
-          pdstbU[p_uv] = (pixel_t)(pdstbU[p_uv] * invInt + interp * RC_f[3 * activeY + 1]); // right half
-          pdstbV[p_uv] = (pixel_t)(pdstbV[p_uv] * invInt + interp * RC_f[3 * activeY + 2]); // right half
+          p_uv = xP + yP * pitchUV;
+          dstp_u[p_uv] = (pixel_t)(dstp_u[p_uv] * invInt + interp * RC_f[3 * activeY + 1]); // right half
+          dstp_v[p_uv] = (pixel_t)(dstp_v[p_uv] * invInt + interp * RC_f[3 * activeY + 2]); // right half
         }
       }
     }
@@ -967,21 +983,24 @@ PVideoFrame Histogram::DrawModeColor2(int n, IScriptEnvironment* env) {
       pdstb, pdstbU, pdstbV,
       dst_pitch, dst_pitchUV,
       dst_height, dst_heightUV,
-      show_bits, swidth, sheight
+      show_bits, swidth, sheight,
+      color2_innerF
       );
   else if (bits_per_pixel <= 16)
     DrawModeColor2_draw_misc<uint16_t>(bits_per_pixel,
       pdstb, pdstbU, pdstbV,
       dst_pitch, dst_pitchUV,
       dst_height, dst_heightUV,
-      show_bits, swidth, sheight
+      show_bits, swidth, sheight,
+      color2_innerF
       );
   else
     DrawModeColor2_draw_misc<float>(bits_per_pixel,
       pdstb, pdstbU, pdstbV,
       dst_pitch, dst_pitchUV,
       dst_height, dst_heightUV,
-      show_bits, swidth, sheight
+      show_bits, swidth, sheight,
+      color2_innerF
       );
 
   // plot white 15 degree marks
@@ -1391,8 +1410,6 @@ PVideoFrame Histogram::DrawModeLevels(int n, IScriptEnvironment* env) {
   const int max_pixel_value = (1 << bits_per_pixel) - 1;
   const float color_shift_factor = isFloat ? 1.0f / 255.0f : max_pixel_value / 255.0f;
 
-  const float middle_chroma_f = uv8tof(128);
-
   int plane_default_black[3] = {
     RGB ? 0 : (16 << color_shift),
     RGB ? 0 : (128 << color_shift),
@@ -1511,7 +1528,6 @@ PVideoFrame Histogram::DrawModeLevels(int n, IScriptEnvironment* env) {
             }
             else {
               // src_offset and dst_offset is 0
-              const int max_src_pixel = (1 << bits_per_pixel) - 1;
               for (int y = 0; y < h; y++) {
                 for (int x = 0; x < w; x++) {
                   hist[min((int)(srcp8[x] * d.mul_factor + 0.5f), max_show_pixel_value)]++;
@@ -1558,7 +1574,6 @@ PVideoFrame Histogram::DrawModeLevels(int n, IScriptEnvironment* env) {
             }
             else {
               // src_offset and dst_offset is 0
-              const int max_src_pixel = (1 << bits_per_pixel) - 1;
               for (int y = 0; y < h; y++) {
                 for (int x = 0; x < w; x++) {
                   hist[min((int)(srcp16[x] * d.mul_factor + 0.5f), max_show_pixel_value)]++;
