@@ -43,69 +43,115 @@
 #include <type_traits>
 
 
-/*******************************
- ********* Masked Blend ********
- *******************************/
-
-AVS_FORCEINLINE static BYTE overlay_blend_c_core_8(const BYTE p1, const BYTE p2, const int mask) {
-  if (mask == 0)
-    return p1;
-  if (mask == 0xFF)
-    return p2;
-  //  p1*(1-mask_f) + p2*mask_f -> p1 + (p2-p1)*mask_f
-  return (BYTE)(((p1 << 8) + (p2 - p1)*mask + 128) >> 8);
-}
-
-template<int bits_per_pixel>
-AVS_FORCEINLINE static uint16_t overlay_blend_c_core_16(const uint16_t p1, const uint16_t p2, const int mask) {
-  if (mask == 0)
-    return p1;
-  if (mask >= (1 << bits_per_pixel) -1)
-    return p2;
-  //  p1*(1-mask_f) + p2*mask_f -> p1 + (p2-p1)*mask_f
-  const int half_rounder = 1 << (bits_per_pixel - 1);
-  if constexpr(bits_per_pixel == 16) // int32 intermediate overflows
-    return (uint16_t)((((int64_t)(p1) << bits_per_pixel) + (p2 - p1)*(int64_t)mask + half_rounder) >> bits_per_pixel);
-  else // signed int arithmetic is enough
-    return (uint16_t)(((p1 << bits_per_pixel) + (p2 - p1)*mask + half_rounder) >> bits_per_pixel);
-}
-
-AVS_FORCEINLINE static float overlay_blend_c_core_f(const float p1, const float p2, const float mask) {
-  return p1 + (p2-p1)*mask; // p1*(1-mask) + p2*mask
-}
-
-
-/*******************************************
- ********* Merge Two Masks Function ********
- *******************************************/
-template<typename pixel_t, typename intermediate_result_t, int bits_per_pixel>
-AVS_FORCEINLINE static pixel_t overlay_merge_mask_c(const pixel_t p1, const pixel_t p2) {
-  return ((intermediate_result_t)p1*p2) >> bits_per_pixel;
-}
-
-AVS_FORCEINLINE static BYTE overlay_merge_mask_c_8(const BYTE p1, const int p2) {
-  return (p1*p2) >> 8;
-}
-
-
-/********************************
- ********* Blend Opaque *********
- ** Use for Lighten and Darken **
- ********************************/
-template<typename pixel_t>
-AVS_FORCEINLINE pixel_t overlay_blend_opaque_c_core(const pixel_t p1, const pixel_t p2, const pixel_t mask) {
-  return (mask) ? p2 : p1;
-}
-
-
 /******************************
  ********* Mode: Blend ********
  ******************************/
 
+// 32 bit float mask calculation inside
+template<bool has_mask, typename pixel_t, int bits_per_pixel>
+void overlay_blend_c_uint(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int opacity, const float opacity_f)
+{
+  const int max_pixel_value = (1 << bits_per_pixel) - 1;
+  auto factor = has_mask ? opacity_f / max_pixel_value : opacity_f;
+
+  for (int y = 0; y < height; y++) {
+    for (int x = 0; x < width; x++) {
+      const float new_mask = has_mask ? (float)reinterpret_cast<const pixel_t*>(mask)[x] * factor : factor;
+      auto result = overlay_blend_c_core_simple(
+        reinterpret_cast<pixel_t*>(p1)[x],
+        reinterpret_cast<const pixel_t*>(p2)[x],
+        new_mask);
+      reinterpret_cast<pixel_t*>(p1)[x] = (pixel_t)(result + 0.5f);
+    }
+
+    p1 += p1_pitch;
+    p2 += p2_pitch;
+    if(has_mask)
+      mask += mask_pitch;
+  }
+}
+
+// instantiate
+// w/o mask
+template void overlay_blend_c_uint<false, uint8_t, 8>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<false, uint16_t, 10>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<false, uint16_t, 12>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<false, uint16_t, 14>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<false, uint16_t, 16>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+// w/ mask
+template void overlay_blend_c_uint<true, uint8_t, 8>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<true, uint16_t, 10>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<true, uint16_t, 12>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<true, uint16_t, 14>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_uint<true, uint16_t, 16>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int opacity, const float opacity_f);
+
+void overlay_blend_c_plane_masked_f(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int /*opacity*/, const float /*opacity_f*/) {
+
+  typedef float pixel_t;
+  for (int y = 0; y < height; y++) {
+    for (int x = 0; x < width; x++) {
+      pixel_t new_mask = reinterpret_cast<const pixel_t*>(mask)[x];
+      pixel_t p1x = reinterpret_cast<pixel_t*>(p1)[x];
+      pixel_t p2x = reinterpret_cast<const pixel_t*>(p2)[x];
+      pixel_t result = p1x + (p2x - p1x) * new_mask; // p1x*(1-new_mask) + p2x*mask
+
+      //pixel_t result = overlay_blend_c_core(reinterpret_cast<pixel_t *>(p1)[x], reinterpret_cast<pixel_t *>(p2)[x], static_cast<int>(reinterpret_cast<pixel_t *>(mask)[x]));
+      reinterpret_cast<pixel_t*>(p1)[x] = result;
+    }
+
+    p1 += p1_pitch;
+    p2 += p2_pitch;
+    mask += mask_pitch;
+  }
+}
+
+template<bool has_mask>
+void overlay_blend_c_float(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int /*opacity*/, const float opacity_f) {
+
+  for (int y = 0; y < height; y++) {
+    for (int x = 0; x < width; x++) {
+      auto new_mask = has_mask ? reinterpret_cast<const float*>(mask)[x] * opacity_f : opacity_f;
+      auto p1x = reinterpret_cast<float*>(p1)[x];
+      auto p2x = reinterpret_cast<const float*>(p2)[x];
+      auto result = p1x + (p2x - p1x) * new_mask; // p1x*(1-new_mask) + p2x*mask
+      reinterpret_cast<float*>(p1)[x] = result;
+    }
+
+    p1 += p1_pitch;
+    p2 += p2_pitch;
+    if constexpr (has_mask)
+      mask += mask_pitch;
+  }
+}
+
+// instantiate
+template void overlay_blend_c_float<false>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int /*opacity*/, const float opacity_f);
+template void overlay_blend_c_float<true>(BYTE* p1, const BYTE* p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch, const int width, const int height, const int /*opacity*/, const float opacity_f);
+
+
+
 template<typename pixel_t, int bits_per_pixel>
 void overlay_blend_c_plane_masked(BYTE *p1, const BYTE *p2, const BYTE *mask,
                                   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-                                  const int width, const int height)
+                                  const int width, const int height, const int /*opacity*/, const float /*opacity_f*/)
 {
   for (int y = 0; y < height; y++) {
     for (int x = 0; x < width; x++) {
@@ -126,50 +172,31 @@ void overlay_blend_c_plane_masked(BYTE *p1, const BYTE *p2, const BYTE *mask,
   }
 }
 
-void overlay_blend_c_plane_masked_f(BYTE *p1, const BYTE *p2, const BYTE *mask,
-  const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height) {
-
-  typedef float pixel_t;
-  for (int y = 0; y < height; y++) {
-    for (int x = 0; x < width; x++) {
-      pixel_t new_mask = reinterpret_cast<const pixel_t *>(mask)[x];
-      pixel_t p1x = reinterpret_cast<pixel_t *>(p1)[x];
-      pixel_t p2x = reinterpret_cast<const pixel_t *>(p2)[x];
-      pixel_t result = p1x + (p2x-p1x)*new_mask; // p1x*(1-new_mask) + p2x*mask
-
-      //pixel_t result = overlay_blend_c_core(reinterpret_cast<pixel_t *>(p1)[x], reinterpret_cast<pixel_t *>(p2)[x], static_cast<int>(reinterpret_cast<pixel_t *>(mask)[x]));
-      reinterpret_cast<pixel_t *>(p1)[x] = result;
-    }
-
-    p1   += p1_pitch;
-    p2   += p2_pitch;
-    mask += mask_pitch;
-  }
-}
 
 // instantiate
 template void overlay_blend_c_plane_masked<uint8_t, 8>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked<uint16_t,10>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height);
+  const int width, const int heigh, const int opacity, const float opacity_ft);
 template void overlay_blend_c_plane_masked<uint16_t,12>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked<uint16_t,14>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked<uint16_t,16>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height);
+  const int width, const int height, const int opacity, const float opacity_f);
 
 
 template<typename pixel_t, int bits_per_pixel>
-void overlay_blend_c_plane_opacity(BYTE *p1, const BYTE *p2,
-                                   const int p1_pitch, const int p2_pitch,
-                                   const int width, const int height, const int opacity) {
+void overlay_blend_c_plane_opacity(BYTE *p1, const BYTE *p2, const BYTE* /*mask*/,
+                                   const int p1_pitch, const int p2_pitch, const int /*mask_pitch*/,
+                                   const int width, const int height, const int opacity, const float opacity_f) {
+
+  AVS_UNUSED(opacity_f);
 
   const int OPACITY_SHIFT  = 8; // opacity always max 0..256
   const int MASK_CORR_SHIFT = OPACITY_SHIFT; // no mask, mask = opacity, 8 bits always
@@ -193,9 +220,11 @@ void overlay_blend_c_plane_opacity(BYTE *p1, const BYTE *p2,
   }
 }
 
-void overlay_blend_c_plane_opacity_f(BYTE *p1, const BYTE *p2,
-  const int p1_pitch, const int p2_pitch,
-  const int width, const int height, const float opacity_f) {
+void overlay_blend_c_plane_opacity_f(BYTE *p1, const BYTE *p2, const BYTE* /*mask*/,
+  const int p1_pitch, const int p2_pitch, const int /*mask_pitch*/,
+  const int width, const int height,const int opacity, const float opacity_f) {
+
+  AVS_UNUSED(opacity);
 
   for (int y = 0; y < height; y++) {
     for (int x = 0; x < width; x++) {
@@ -211,27 +240,30 @@ void overlay_blend_c_plane_opacity_f(BYTE *p1, const BYTE *p2,
 }
 
 // instantiate
-template void overlay_blend_c_plane_opacity<uint8_t, 8>(BYTE *p1, const BYTE *p2,
-  const int p1_pitch, const int p2_pitch,
-  const int width, const int height, const int opacity);
-template void overlay_blend_c_plane_opacity<uint16_t,10>(BYTE *p1, const BYTE *p2,
-  const int p1_pitch, const int p2_pitch,
-  const int width, const int height, const int opacity);
-template void overlay_blend_c_plane_opacity<uint16_t,12>(BYTE *p1, const BYTE *p2,
-  const int p1_pitch, const int p2_pitch,
-  const int width, const int height, const int opacity);
-template void overlay_blend_c_plane_opacity<uint16_t,14>(BYTE *p1, const BYTE *p2,
-  const int p1_pitch, const int p2_pitch,
-  const int width, const int height, const int opacity);
-template void overlay_blend_c_plane_opacity<uint16_t,16>(BYTE *p1, const BYTE *p2,
-  const int p1_pitch, const int p2_pitch,
-  const int width, const int height, const int opacity);
+template void overlay_blend_c_plane_opacity<uint8_t, 8>(BYTE *p1, const BYTE *p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_plane_opacity<uint16_t,10>(BYTE *p1, const BYTE *p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_plane_opacity<uint16_t,12>(BYTE *p1, const BYTE *p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_plane_opacity<uint16_t,14>(BYTE *p1, const BYTE *p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int opacity, const float opacity_f);
+template void overlay_blend_c_plane_opacity<uint16_t,16>(BYTE *p1, const BYTE *p2, const BYTE* mask,
+  const int p1_pitch, const int p2_pitch, const int mask_pitch,
+  const int width, const int height, const int opacity, const float opacity_f);
 
 
 template<typename pixel_t, int bits_per_pixel>
 void overlay_blend_c_plane_masked_opacity(BYTE *p1, const BYTE *p2, const BYTE *mask,
                                   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-                                  const int width, const int height, const int opacity) {
+                                  const int width, const int height, const int opacity, const float opacity_f) {
+
+  AVS_UNUSED(opacity_f);
+
   const int MASK_CORR_SHIFT = (sizeof(pixel_t) == 1) ? 8 : bits_per_pixel;
   const int OPACITY_SHIFT  = 8; // opacity always max 0..256
   const int half_pixel_value_rounding = (1 << (MASK_CORR_SHIFT - 1));
@@ -260,23 +292,25 @@ void overlay_blend_c_plane_masked_opacity(BYTE *p1, const BYTE *p2, const BYTE *
 // instantiate
 template void overlay_blend_c_plane_masked_opacity<uint8_t, 8>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height, const int opacity);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked_opacity<uint16_t,10>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height, const int opacity);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked_opacity<uint16_t,12>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height, const int opacity);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked_opacity<uint16_t,14>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height, const int opacity);
+  const int width, const int height, const int opacity, const float opacity_f);
 template void overlay_blend_c_plane_masked_opacity<uint16_t,16>(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height, const int opacity);
+  const int width, const int height, const int opacity, const float opacity_f);
 
 void overlay_blend_c_plane_masked_opacity_f(BYTE *p1, const BYTE *p2, const BYTE *mask,
   const int p1_pitch, const int p2_pitch, const int mask_pitch,
-  const int width, const int height, const float opacity_f) {
+  const int width, const int height, const int opacity, const float opacity_f) {
+
+  AVS_UNUSED(opacity);
 
   for (int y = 0; y < height; y++) {
     for (int x = 0; x < width; x++) {
@@ -334,16 +368,6 @@ AVS_FORCEINLINE void overlay_darklighten_c(BYTE *p1Y_8, BYTE *p1U_8, BYTE *p1V_8
   }
 }
 
-// Compare functions for lighten and darken mode
-AVS_FORCEINLINE int overlay_darken_c_cmp(BYTE p1, BYTE p2) {
-  return p2 <= p1;
-}
-
-template<typename pixel_t>
-AVS_FORCEINLINE int overlay_lighten_c_cmp(pixel_t p1, pixel_t p2) {
-  return p2 >= p1;
-}
-
 // Exported function
 template<typename pixel_t>
 void overlay_darken_c(BYTE *p1Y_8, BYTE *p1U_8, BYTE *p1V_8, const BYTE *p2Y_8, const BYTE *p2U_8, const BYTE *p2V_8, int p1_pitch, int p2_pitch, int width, int height) {
