@@ -277,16 +277,6 @@ SwapUVToY::SwapUVToY(PClip _child, int _mode, IScriptEnvironment* env)
   }
 }
 
-template <typename T>
-static void fill_plane(BYTE* dstp, int rowsize, int height, int pitch, T val)
-{
-  rowsize /= sizeof(T);
-  for (int y = 0; y < height; ++y) {
-    std::fill_n(reinterpret_cast<T*>(dstp), rowsize, val);
-    dstp += pitch;
-  }
-}
-
 PVideoFrame __stdcall SwapUVToY::GetFrame(int n, IScriptEnvironment* env)
 {
   PVideoFrame src = child->GetFrame(n, env);
@@ -328,7 +318,7 @@ PVideoFrame __stdcall SwapUVToY::GetFrame(int n, IScriptEnvironment* env)
     int dst_pitch = dst->GetPitch();
     int pos = (mode == YUY2UToY8 || mode == UToY) ? 1 : 3; // YUYV U=offset#1 V=offset#3
 
-    if (vi.IsYUY2()) {  // YUY2 To YUY2
+    if (vi.IsYUY2()) {  // YUY2 to YUY2
       int rowsize = dst->GetRowSize();
 #ifdef INTEL_INTRINSICS
       if (env->GetCPUFlags() & CPUF_SSE2) {
@@ -385,18 +375,15 @@ PVideoFrame __stdcall SwapUVToY::GetFrame(int n, IScriptEnvironment* env)
   BYTE* dstp_v = dst->GetWritePtr(PLANAR_V);
 
   if (vi.ComponentSize() == 1) {  // 8bit
-    fill_plane<BYTE>(dstp_u, rowsize, height, pitch, 0x80);
-    fill_plane<BYTE>(dstp_v, rowsize, height, pitch, 0x80);
+    fill_chroma<BYTE>(dstp_u, dstp_v, height, rowsize, pitch, 0x80);
   }
   else if (vi.ComponentSize() == 2) {  // 16bit
     uint16_t grey_val = 1 << (vi.BitsPerComponent() - 1); // 0x8000 for 16 bit
-    fill_plane<uint16_t>(dstp_u, rowsize, height, pitch, grey_val);
-    fill_plane<uint16_t>(dstp_v, rowsize, height, pitch, grey_val);
+    fill_chroma<uint16_t>(dstp_u, dstp_v, height, rowsize, pitch, grey_val);
   }
   else {  // 32bit(float)
     float grey_val = uv8tof(128);
-    fill_plane<float>(dstp_u, rowsize, height, pitch, grey_val);
-    fill_plane<float>(dstp_v, rowsize, height, pitch, grey_val);
+    fill_chroma<float>(dstp_u, dstp_v, height, rowsize, pitch, grey_val);
   }
 
   return dst;
@@ -877,17 +864,23 @@ PVideoFrame __stdcall CombinePlanes::GetFrame(int n, IScriptEnvironment* env) {
       //  3010       2010        1010       10      new offsets inside
     }
 
+    PVideoFrame dst;
     if (vi.NumComponents() == 4) {
-      return env->SubframePlanarA(src, RelOffsets[0], NewPitches[0], NewRowSizes[0], src->GetHeight(),
+      dst = env->SubframePlanarA(src, RelOffsets[0], NewPitches[0], NewRowSizes[0], src->GetHeight(),
         RelOffsets[1], RelOffsets[2], NewPitches[1], RelOffsets[3]);
     }
     else if (vi.NumComponents() == 3) {
-      return env->SubframePlanar(src, RelOffsets[0], NewPitches[0], NewRowSizes[0], src->GetHeight(),
+      dst = env->SubframePlanar(src, RelOffsets[0], NewPitches[0], NewRowSizes[0], src->GetHeight(),
         RelOffsets[1], RelOffsets[2], NewPitches[1]);
     }
     else {
-      return env->Subframe(src, RelOffsets[0], NewPitches[0], NewRowSizes[0], src->GetHeight());
+      dst = env->Subframe(src, RelOffsets[0], NewPitches[0], NewRowSizes[0], src->GetHeight());
     }
+
+    // RGB(A)<->YUV(A) color space conversion can't be caught by Subframe...()
+    dst->AmendPixelType(vi.pixel_type);
+
+    return dst;
   }
 
   // check if first clip could be used as the target clip
@@ -900,29 +893,35 @@ PVideoFrame __stdcall CombinePlanes::GetFrame(int n, IScriptEnvironment* env) {
     // luma (Y) comes w/o BitBlt. Only U and V (and optionally A) is copied.
     if (src->IsWritable()) // we are the only one
     {
+      src->AmendPixelType(vi.pixel_type);
+
       PVideoFrame src_other = nullptr;
       bool writeptr_obtained = false;
+
       for (int i = 1; i < planecount; i++) {
         int target_plane = target_planes[i];
         int source_plane = source_planes[i];
+
         if (clips[i]) { // source clips can be less than defined planes
           if (!writeptr_obtained) {
             src->GetWritePtr(PLANAR_Y); //Must be requested BUT only if we actually do something
             writeptr_obtained = true;
           }
+
           if (i == 1)
             src_other = src1; // already requested
           else
             src_other = clips[i]->GetFrame(n, env); // last defined clip is used for the others
         }
+
         if (src_other) {
           env->BitBlt(src->GetWritePtr(target_plane), src->GetPitch(target_plane),
             src_other->GetReadPtr(source_plane), src_other->GetPitch(source_plane), src_other->GetRowSize(source_plane), src_other->GetHeight(source_plane));
-        }
-        else {
+        } else {
           // we are still at the first (master) clip, no need for plane copy
         }
       }
+
       return src;
     }
   }
@@ -946,13 +945,19 @@ PVideoFrame __stdcall CombinePlanes::GetFrame(int n, IScriptEnvironment* env) {
     {
       if (src1->IsWritable()) // we are the only one
       {
+        src1->AmendPixelType(vi.pixel_type);
+
         src1->GetWritePtr(PLANAR_Y); //Must be requested BUT only if we actually do something
+
         int target_plane = target_planes[0];
         int source_plane = source_planes[0];
+
         // Copy from first clip
         env->BitBlt(src1->GetWritePtr(target_plane), src1->GetPitch(target_plane),
           src->GetReadPtr(source_plane), src->GetPitch(source_plane), src->GetRowSize(source_plane), src->GetHeight(source_plane));
+
         env->copyFrameProps(src, src1);
+
         return src1;
       }
     }
@@ -963,22 +968,25 @@ PVideoFrame __stdcall CombinePlanes::GetFrame(int n, IScriptEnvironment* env) {
 
   for (int i = 0; i < planecount; i++) {
     if (clips[i]) { // source clips can be less than defined planes
-      if (i > 0) // clip #0 was already requested
-      {
+      if (i > 0) { // clip #0 was already requested
         if (i == 1) // clip #1 was already requested
           src = src1;
         else
           src = clips[i]->GetFrame(n, env); // last defined clip is used for the others
       }
+  
       if (!propCopied) {
         env->copyFrameProps(src, dst);
         propCopied = true;
       }
     }
+
     int target_plane = target_planes[i];
     int source_plane = source_planes[i];
+
     env->BitBlt(dst->GetWritePtr(target_plane), dst->GetPitch(target_plane),
       src->GetReadPtr(source_plane), src->GetPitch(source_plane), src->GetRowSize(source_plane), src->GetHeight(source_plane));
   }
+
   return dst;
 }
