@@ -157,12 +157,13 @@ static PVideoFrame CreateBlankFrame(const VideoInfo& vi, int color, int mode, co
       {
         int plane = planes[p];
         BYTE *dstp = frame->GetWritePtr(plane);
+        int rowsize = frame->GetRowSize(plane);
         int pitch = frame->GetPitch(plane);
         int height = frame->GetHeight(plane);
         switch (pixelsize) {
-        case 1: fill_plane<uint8_t>(dstp, height, pitch, clamp(colors[p], 0, 0xFF)); break;
-        case 2: fill_plane<uint16_t>(dstp, height, pitch, clamp(colors[p], 0, (1 << vi.BitsPerComponent()) - 1)); break;
-        case 4: fill_plane<float>(dstp, height, pitch, colors_f[p]); break;
+        case 1: fill_plane<uint8_t>(dstp, height, rowsize, pitch, clamp(colors[p], 0, 0xFF)); break;
+        case 2: fill_plane<uint16_t>(dstp, height, rowsize, pitch, clamp(colors[p], 0, (1 << vi.BitsPerComponent()) - 1)); break;
+        case 4: fill_plane<float>(dstp, height, rowsize, pitch, colors_f[p]); break;
         }
       }
     }
@@ -507,13 +508,13 @@ PClip Create_MessageClip(const char* message, int width, int height, int pixel_t
                          IScriptEnvironment* env) {
   int size;
 #if defined(AVS_WINDOWS) && !defined(NO_WIN_GDI)
-    // MessageClip produces a clip containing a text message.Used internally for error reporting.
-    // The font face is "Arial".
-    // The font size is between 24 points and 9 points - chosen to fit, if possible,
-    // in the width by height clip.
-    // The pixeltype is RGB32.
+  // MessageClip produces a clip containing a text message.Used internally for error reporting.
+  // The font face is "Arial".
+  // The font size is between 24 points and 9 points - chosen to fit, if possible,
+  // in the width by height clip.
+  // The pixeltype is RGB32.
 
-    for (size = 24*8; /*size>=9*8*/; size-=4) {
+  for (size = 24*8; /*size>=9*8*/; size-=4) {
     int text_width, text_height;
     GetTextBoundingBox(message, "Arial", size, true, false, TA_TOP | TA_CENTER, &text_width, &text_height);
     text_width = ((text_width>>3)+8+7) & ~7; // mod 8
@@ -1589,14 +1590,14 @@ public:
     }
     update_Matrix_and_ColorRange(props, theMatrix, theColorRange, env);
 
-	// HD colorbars arib_std_b28
-	// Rec709 yuv values calculated by jmac698, Jan 2010, for Midzuki
-	if (type) { // ColorbarsHD
-		BYTE* pY = (BYTE*)frame->GetWritePtr(PLANAR_Y);
-		BYTE* pU = (BYTE*)frame->GetWritePtr(PLANAR_U);
-		BYTE* pV = (BYTE*)frame->GetWritePtr(PLANAR_V);
-		const int pitchY  = frame->GetPitch(PLANAR_Y);
-		const int pitchUV = frame->GetPitch(PLANAR_U);
+  // HD colorbars arib_std_b28
+  // Rec709 yuv values calculated by jmac698, Jan 2010, for Midzuki
+  if (type) { // ColorbarsHD
+    BYTE* pY = (BYTE*)frame->GetWritePtr(PLANAR_Y);
+    BYTE* pU = (BYTE*)frame->GetWritePtr(PLANAR_U);
+    BYTE* pV = (BYTE*)frame->GetWritePtr(PLANAR_V);
+    const int pitchY  = frame->GetPitch(PLANAR_Y);
+    const int pitchUV = frame->GetPitch(PLANAR_U);
 
     switch (bits_per_pixel) {
     case 8: draw_colorbarsHD_444<uint8_t, 8>(pY, pU, pV, pitchY, pitchUV, w, h); break;
@@ -1607,8 +1608,8 @@ public:
     case 32: draw_colorbarsHD_444<float, 8 /* n/a */>(pY, pU, pV, pitchY, pitchUV, w, h); break;
     }
 
-	}
-	// Rec. ITU-R BT.801-1
+  }
+  // Rec. ITU-R BT.801-1
   else if (vi.Is444()) {
     BYTE* pY = (BYTE*)frame->GetWritePtr(PLANAR_Y);
     BYTE* pU = (BYTE*)frame->GetWritePtr(PLANAR_U);
@@ -1624,7 +1625,7 @@ public:
     case 16: draw_colorbars_444<uint16_t, 16>(pY, pU, pV, pitchY, pitchUV, w, h); break;
     case 32: draw_colorbars_444<float, 8 /* n/a */>(pY, pU, pV, pitchY, pitchUV, w, h); break;
     }
-	}
+  }
   else if (vi.IsRGB() && vi.IsPlanar()) {
     BYTE* pG = (BYTE*)frame->GetWritePtr(PLANAR_G);
     BYTE* pB = (BYTE*)frame->GetWritePtr(PLANAR_B);
@@ -1646,7 +1647,7 @@ public:
     case 8: draw_colorbars_rgb3264<uint8_t>((uint8_t *)p, pitch, w, h); break;
     case 16: draw_colorbars_rgb3264<uint16_t>((uint8_t *)p, pitch, w, h); break;
     }
-	}
+  }
   else if (vi.IsRGB24() || vi.IsRGB48()) {
     const int pitch = frame->GetPitch();
     switch (bits_per_pixel) {
@@ -1748,12 +1749,13 @@ public:
   if (vi.IsYUVA() || vi.IsPlanarRGBA()) {
     // initialize planar alpha planes with zero (no transparency), like RGB32 does
     BYTE *dstp = frame->GetWritePtr(PLANAR_A);
+    int rowsize = frame->GetRowSize(PLANAR_A);
     int pitch = frame->GetPitch(PLANAR_A);
     int height = frame->GetHeight(PLANAR_A);
     switch (bits_per_pixel) {
-    case 8: fill_plane<uint8_t>(dstp, height, pitch, 0); break;
-    case 10: case 12: case 14: case 16: fill_plane<uint16_t>(dstp, height, pitch, 0); break;
-    case 32: fill_plane<float>(dstp, height, pitch, 0.0f); break;
+    case 8: fill_plane<uint8_t>(dstp, height, rowsize, pitch, 0); break;
+    case 10: case 12: case 14: case 16: fill_plane<uint16_t>(dstp, height, rowsize, pitch, 0); break;
+    case 32: fill_plane<float>(dstp, height, rowsize, pitch, 0.0f); break;
     }
   }
 
@@ -1843,19 +1845,19 @@ public:
       count = (vi.num_audio_samples - start);
     }
 
-	const int d_mod = vi.audio_samples_per_second*2;
-	float* samples = (float*)buf;
+  const int d_mod = vi.audio_samples_per_second*2;
+  float* samples = (float*)buf;
 
-	unsigned j = (unsigned)(start % nsamples);
-	for (int i=0;i<count;i++) {
-	  samples[i*2]=audio[j];
-	  if (((start+i)%d_mod)>vi.audio_samples_per_second) {
-		samples[i*2+1]=audio[j];
-	  } else {
-		samples[i*2+1]=0;
-	  }
-	  if (++j >= nsamples) j = 0;
-	}
+  unsigned j = (unsigned)(start % nsamples);
+  for (int i=0;i<count;i++) {
+    samples[i*2]=audio[j];
+    if (((start+i)%d_mod)>vi.audio_samples_per_second) {
+    samples[i*2+1]=audio[j];
+    } else {
+    samples[i*2+1]=0;
+    }
+    if (++j >= nsamples) j = 0;
+  }
 #else
     int64_t Hz=440;
     // Calculate what start equates in cycles.
@@ -2118,7 +2120,7 @@ public:
 
   static AVSValue __cdecl Create(AVSValue args, void*, IScriptEnvironment* env)
   {
-	  return new Tone(args[0].AsFloat(10.0f), args[1].AsFloat(440.0f), args[2].AsInt(48000),
+    return new Tone(args[0].AsFloat(10.0f), args[1].AsFloat(440.0f), args[2].AsInt(48000),
           args[3].AsInt(2), args[4].AsString("Sine"), args[5].AsFloatf(1.0f), env);
   }
 
@@ -2171,9 +2173,9 @@ AVSValue __cdecl Create_Version(AVSValue args, void*, IScriptEnvironment* env) {
 
   return Create_MessageClip(
 #ifdef AVS_POSIX
-    AVS_FULLVERSION AVS_COPYRIGHT_UTF8
+    AVS_FULLVERSION AVS_DEVELOPMENT_BUILD AVS_DEVELOPMENT_BUILD_GIT AVS_COPYRIGHT_UTF8
 #else
-    AVS_FULLVERSION AVS_COPYRIGHT
+    AVS_FULLVERSION AVS_DEVELOPMENT_BUILD AVS_DEVELOPMENT_BUILD_GIT AVS_COPYRIGHT
 #endif
     , w, h, i_pixel_type, shrink, textcolor, halocolor, bgcolor, fps_numerator, fps_denominator, num_frames, env);
 }
