@@ -60,8 +60,8 @@ class Antialiaser
  **/
 {
 public:
-  Antialiaser(int width, int height, const char fontname[], int size,
-	  int textcolor, int halocolor, int font_width=0, int font_angle=0, bool _interlaced=false);
+  Antialiaser(int width, int height, const char fontname[], int size, 
+    int textcolor, int halocolor, bool _bold, bool _italic, bool _noaa, int font_width=0, int font_angle=0, bool _interlaced=false);
   virtual ~Antialiaser();
   HDC GetDC();
   void FreeDC();
@@ -74,8 +74,9 @@ private:
   void ApplyPlanar_core(BYTE* buf, int pitch, int UVpitch,BYTE* bufV,BYTE* bufU,bool isRGB);
   void ApplyPlanar(BYTE* buf, int pitch, int UVpitch,BYTE* bufV,BYTE* bufU, int shiftX, int shiftY, int pixelsize, bool isRGB);
   void ApplyYUY2(BYTE* buf, int pitch);
-  void ApplyRGB24_48(BYTE* buf, int pitch, int pixelsize);
-  void ApplyRGB32_64(BYTE* buf, int pitch, int pixelsize);
+
+  template<typename pixel_t, bool has_alpha>
+  void ApplyRGB_packed(BYTE* buf, int pitch);
 
   void* lpAntialiasBits;
   unsigned short* alpha_calcs;
@@ -87,6 +88,8 @@ private:
   const int textcolor, halocolor;
   int xl, yt, xr, yb; // sub-rectangle containing live text
   bool dirty, interlaced;
+  bool bold, italic;
+  bool noaa;
 
   void GetAlphaRect();
 };
@@ -100,7 +103,7 @@ class ShowFrameNumber : public GenericVideoFilter
 {
 public:
   ShowFrameNumber(PClip _child, bool _scroll, int _offset, int _x, int _y, const char _fontname[], int _size,
-			int _textcolor, int _halocolor, int font_width, int font_angle, IScriptEnvironment* env);
+      int _textcolor, int _halocolor, int font_width, int font_angle, bool _bold, bool _italic, bool _noaa, IScriptEnvironment* env);
   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) override;
 
   static AVSValue __cdecl Create(AVSValue args, void*, IScriptEnvironment* env);
@@ -116,11 +119,15 @@ private:
   Antialiaser antialiaser;
 #else
   std::unique_ptr<BitmapFont> current_font;
+  int chromaplacement;
 #endif
   const bool scroll;
   const int offset;
   const int size, x, y;
   const int textcolor, halocolor;
+  const bool bold;
+  const bool italic; // n/a in NO_WIN_GDI
+  const bool noaa; // n/a in NO_WIN_GDI
 };
 
 class ShowCRC32 : public GenericVideoFilter
@@ -134,7 +141,7 @@ class ShowCRC32 : public GenericVideoFilter
 
 public:
   ShowCRC32(PClip _child, bool _scroll, int _offset, int _x, int _y, const char _fontname[], int _size,
-    int _textcolor, int _halocolor, int font_width, int font_angle, IScriptEnvironment* env);
+            int _textcolor, int _halocolor, int font_width, int font_angle, bool _bold, bool _italic, bool _noaa, IScriptEnvironment* env);
   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) override;
 
   static AVSValue __cdecl Create(AVSValue args, void*, IScriptEnvironment* env);
@@ -150,11 +157,15 @@ private:
   Antialiaser antialiaser;
 #else
   std::unique_ptr<BitmapFont> current_font;
+  int chromaplacement;
 #endif
   const bool scroll;
   const int offset;
   const int size, x, y;
   const int textcolor, halocolor;
+  const bool bold;
+  const bool italic; // n/a in NO_WIN_GDI
+  const bool noaa; // n/a in NO_WIN_GDI
 };
 
 
@@ -166,7 +177,7 @@ class ShowSMPTE : public GenericVideoFilter
 {
 public:
   ShowSMPTE(PClip _child, double _rate, const char* _offset, int _offset_f, int _x, int _y, const char _fontname[], int _size,
-			int _textcolor, int _halocolor, int font_width, int font_angle, IScriptEnvironment* env);
+            int _textcolor, int _halocolor, int font_width, int font_angle, bool _bold, bool _italic, bool _noaa, IScriptEnvironment* env);
   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) override;
 
   static AVSValue __cdecl CreateSMTPE(AVSValue args, void*, IScriptEnvironment* env);
@@ -183,12 +194,16 @@ private:
   Antialiaser antialiaser;
 #else
   std::unique_ptr<BitmapFont> current_font;
+  int chromaplacement;
 #endif
   int rate;
   int offset_f;
   const int x, y;
   bool dropframe;
   const int textcolor, halocolor;
+  const bool bold;
+  const bool italic; // n/a in NO_WIN_GDI
+  const bool noaa; // n/a in NO_WIN_GDI
 };
 
 
@@ -202,7 +217,8 @@ class Subtitle : public GenericVideoFilter
 public:
   Subtitle( PClip _child, const char _text[], int _x, int _y, int _firstframe, int _lastframe,
             const char _fontname[], int _size, int _textcolor, int _halocolor, int _align,
-            int _spc, bool _multiline, int _lsp, int _font_width, int _font_angle, bool _interlaced, const char _font_filename[], const bool _utf8, IScriptEnvironment* env);
+            int _spc, bool _multiline, int _lsp, int _font_width, int _font_angle, bool _interlaced, const char _font_filename[], const bool _utf8,
+            const bool _bold, const bool _italic, const bool _noaa, IScriptEnvironment* env);
   virtual ~Subtitle(void);
   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) override;
 
@@ -224,6 +240,9 @@ private:
   const char* const text;
   const char* const font_filename;
   const bool utf8;
+  const bool bold;
+  const bool italic;
+  const bool noaa;
   Antialiaser* antialiaser;
 };
 #endif
@@ -236,7 +255,9 @@ class SimpleText : public GenericVideoFilter
 public:
   SimpleText(PClip _child, const char _text[], int _x, int _y, int _firstframe, int _lastframe,
     const char _fontname[], int _size, int _textcolor, int _halocolor, int _align,
-    int _spc, bool _multiline, int _lsp, int _font_width, int _font_angle, bool _interlaced, const char _font_filename[], const bool _utf8, const bool _bold, IScriptEnvironment* env);
+    int _spc, bool _multiline, int _lsp, int _font_width, int _font_angle, bool _interlaced, const char _font_filename[],
+    const bool _utf8, const bool _bold, const int _chromalocation,
+    IScriptEnvironment* env);
   virtual ~SimpleText(void);
   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) override;
 
@@ -262,6 +283,7 @@ private:
   const char* const font_filename; // .BDF files
   const bool utf8;
   const bool bold;
+  const int chromalocation;
   std::unique_ptr<BitmapFont> current_font;
 };
 
@@ -271,7 +293,7 @@ class FilterInfo : public GenericVideoFilter
  **/
 {
 public:
-  FilterInfo( PClip _child, const char _fontname[], int _size, int _textcolor, int _halocolor, IScriptEnvironment* env);
+  FilterInfo( PClip _child, const char _fontname[], int _size, int _textcolor, int _halocolor, bool _bold, bool _italic, bool _noaa, IScriptEnvironment* env);
   virtual ~FilterInfo(void);
   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env) override;
   bool __stdcall GetParity(int n) override;
@@ -296,7 +318,11 @@ private:
   Antialiaser antialiaser;
 #else
   std::unique_ptr<BitmapFont> current_font;
+  int chromaplacement;
 #endif
+  const bool bold;
+  const bool italic;
+  const bool noaa;
 };
 
 
@@ -327,6 +353,7 @@ private:
   Antialiaser antialiaser;
 #else
   std::unique_ptr<BitmapFont> current_font;
+  int chromaplacement;
 #endif
   PClip child2;
   uint32_t mask;
