@@ -685,73 +685,6 @@ AVSValue __cdecl AddBorders::Create(AVSValue args, void*, IScriptEnvironment* en
 }
 
 
-
-
-
-/******************************
- *******   Fill Border   ******
- *****************************/
-
-
- /*  This function fills up the right side of the picture on planar images with duplicates of the rightmost pixel
-  *   TODO: Implement fast ISSE routines
-  */
-
-// PF dead function?
-FillBorder::FillBorder(PClip _clip) : GenericVideoFilter(_clip) {
-}
-
-PVideoFrame __stdcall FillBorder::GetFrame(int n, IScriptEnvironment* env) {
-
-  PVideoFrame src = child->GetFrame(n, env);
-  if (src->GetRowSize(PLANAR_Y)==src->GetRowSize(PLANAR_Y_ALIGNED)) return src;  // No need to fill extra pixels
-
-  // !! when GetWritePtr would be size_t, revise this. Be cautious when you subtract two unsigned size_t variables
-  unsigned char* Ydata = src->GetWritePtr(PLANAR_U) - (src->GetOffset(PLANAR_U) - src->GetOffset(PLANAR_Y)); // Nasty hack, to avoid "MakeWritable" - never, EVER do this at home!
-  unsigned char* Udata = src->GetWritePtr(PLANAR_U);
-  unsigned char* Vdata = src->GetWritePtr(PLANAR_V);
-
-  int fillp=src->GetRowSize(PLANAR_Y_ALIGNED) - src->GetRowSize(PLANAR_Y);
-  int h=src->GetHeight(PLANAR_Y);
-
-  Ydata = &Ydata[src->GetRowSize(PLANAR_Y)-1];
-  for (int y=0; y<h; y++) {
-    for (int x=1; x<=fillp; x++) {
-      Ydata[x]=Ydata[0];
-    }
-    Ydata+=src->GetPitch(PLANAR_Y);
-  }
-
-  fillp=src->GetRowSize(PLANAR_U_ALIGNED) - src->GetRowSize(PLANAR_U);
-  Udata = &Udata[src->GetRowSize(PLANAR_U)-1];
-  Vdata = &Vdata[src->GetRowSize(PLANAR_V)-1];
-  h=src->GetHeight(PLANAR_U);
-
-  for (int y=0; y<h; y++) {
-    for (int x=1; x<=fillp; x++) {
-      Udata[x]=Udata[0];
-      Vdata[x]=Vdata[0];
-    }
-    Udata+=src->GetPitch(PLANAR_U);
-    Vdata+=src->GetPitch(PLANAR_V);
-  }
-  return src;
-}
-
-
-PClip FillBorder::Create(PClip clip)
-{
-  if (!clip->GetVideoInfo().IsPlanar()) {  // If not planar, already ok.
-    return clip;
-  }
-  else
-    return new FillBorder(clip);
-}
-
-
-
-
-
 /**********************************
  *******   Factory Methods   ******
  *********************************/
